package de.fernunihagen.rtt.framework.repository.team.svn;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IResource;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;

import org.eclipse.team.svn.core.operation.IActionOperation;
import org.eclipse.team.svn.core.operation.local.CreatePatchOperation;
import org.eclipse.team.svn.core.operation.local.RefreshResourcesOperation;
import org.eclipse.team.svn.core.operation.local.RevertOperation;

import org.eclipse.team.svn.core.utility.ProgressMonitorUtility;

import de.fernunihagen.rtt.framework.repository.team.TeamRepository;

/**
 * Team svn respository.
 * 
 * @author Osama El Hosami
 */
@Contract
public class Repository extends TeamRepository {

	/**
	 * Repository type identifier
	 */
	public static final String ID = "org.eclipse.team.svn.core.svnnature";

	/**
	 * Creates a svn team repository for given repository provider type
	 * identifier
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             <ul>
	 */
	public Repository() throws CoreException {
		super(ID);
	}

	@Override
	@Pre
	public void revertChanges(final IResource[] resources,
			final boolean fRecurse, final IProgressMonitor pm)
			throws CoreException {

		try {
			final SubMonitor monitor = SubMonitor.convert(pm);
			final RevertOperation revertOp = new RevertOperation(resources,
					fRecurse);

			ProgressMonitorUtility.doTask(revertOp, monitor, 100, 0);

			if (revertOp.getExecutionState() != IActionOperation.OK) {
				throw new CoreException(revertOp.getStatus());
			}
		} finally {
			if (null != pm) {
				pm.done();
			}
		}
	}

	/**
	 * Creates a patch for given resources.
	 * 
	 * @param resources
	 *            Resources
	 * @param fileName
	 *            File name
	 * @param fRecurse
	 *            Flag indicate recursion is requested
	 * @param pm
	 *            Progress monitor
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 * 
	 * @deprecated
	 */
	@Pre("null != resources")
	public void createPatch(final IResource[] resources, final String fileName,
			final boolean fRecurse, final IProgressMonitor pm)
			throws CoreException {

		try {
			final SubMonitor monitor = SubMonitor.convert(pm);
			final CreatePatchOperation patchOp = new CreatePatchOperation(
					resources, fileName, fRecurse, false, false, true,
					CreatePatchOperation.SELECTION);

			ProgressMonitorUtility.doTask(patchOp, monitor, 100, 0);

			if (patchOp.getExecutionState() != IActionOperation.OK) {
				throw new CoreException(patchOp.getStatus());
			}
		} finally {
			if (null != pm) {
				pm.done();
			}
		}
	}

	@Override
	@Pre
	public void refreshLocal(final IResource[] resources, final int depth,
			final IProgressMonitor pm) throws CoreException {

		try {
			final SubMonitor monitor = SubMonitor.convert(pm);
			final RefreshResourcesOperation refreshOp = new RefreshResourcesOperation(
					resources, depth, RefreshResourcesOperation.REFRESH_ALL);

			ProgressMonitorUtility.doTask(refreshOp, monitor, 100, 0);

			if (refreshOp.getExecutionState() != IActionOperation.OK) {
				throw new CoreException(refreshOp.getStatus());
			}
		} finally {
			if (null != pm) {
				pm.done();
			}
		}
	}
}
