package de.fernunihagen.rtt.ui.app;

import static de.fernunihagen.rtt.ui.app.internal.RTTAppUIPlugin.getRTTAppUIPlugin;

import java.lang.reflect.InvocationTargetException;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.IPlatformRunnable;
import org.eclipse.core.runtime.IProgressMonitor;

import org.eclipse.core.runtime.IProduct;
import org.eclipse.core.runtime.Platform;

import org.eclipse.equinox.app.IApplication;
import org.eclipse.equinox.app.IApplicationContext;

import org.eclipse.jface.dialogs.ProgressMonitorDialog;

import org.eclipse.jface.operation.IRunnableWithProgress;

import org.eclipse.swt.widgets.Display;

import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.PlatformUI;

import org.eclipse.ui.testing.ITestHarness;
import org.eclipse.ui.testing.TestableObject;

import de.fernunihagen.rtt.framework.ITestApplication;

/**
 * Abstract test UI workbench, used to run tests on startup. Implementation
 * based on {@link org.eclipse.pde.internal.junit.runtime.UITestApplication}.
 * 
 * @author Osama El Hosami
 */
@Contract
@SuppressWarnings("deprecation")
public abstract class AbstractUITestBench implements IApplication, ITestHarness {

	private static final String DEFAULT_APP_3_0 = "org.eclipse.ui.ide.workbench";

	private TestableObject _testableObject;
	private IApplication _application;

	@Override
	@Pre("null != context")
	public Object start(final IApplicationContext context) throws Exception {
		String[] args = (String[]) context.getArguments().get(
				IApplicationContext.APPLICATION_ARGS);
		Object app = getApplication(args);

		Assert.isNotNull(app);

		_testableObject = PlatformUI.getTestableObject();
		_testableObject.setTestHarness(this);

		if (app instanceof IApplication) {
			_application = (IApplication) app;
			return _application.start(context);
		}

		return ((IPlatformRunnable) app).run(args);
	}

	@Override
	public void stop() {
		if (_application != null) {
			_application.stop();
		}
	}

	@Pre("null != args")
	private Object getApplication(final String[] args) throws CoreException {
		// Find the name of the application as specified by the launcher.
		// If no application is specified, the 3.0 default workbench application
		// is returned.
		final IExtension extension = Platform.getExtensionRegistry()
				.getExtension(Platform.PI_RUNTIME, Platform.PT_APPLICATIONS,
						getApplicationToRun(args));

		Assert.isNotNull(extension);

		// If the extension does not have the correct grammar, return null.
		// Otherwise, return the application object.
		final IConfigurationElement[] elements = extension
				.getConfigurationElements();
		if (elements.length > 0) {
			final IConfigurationElement[] runs = elements[0].getChildren("run"); //$NON-NLS-1$
			if (runs.length > 0) {
				final Object runnable = runs[0]
						.createExecutableExtension("class"); //$NON-NLS-1$				
				if (runnable instanceof IPlatformRunnable
						|| runnable instanceof IApplication)
					return runnable;
			}
		}
		return null;
	}

	/**
	 * The -testApplication argument specifies the application to be run. If the
	 * launcher did not set this argument, then return the name of the default
	 * application. In 3.0, the default is the "org.eclipse.ui.ide.worbench"
	 * application.
	 * 
	 * @param args
	 *            Program arguments
	 */
	@Pre("null != args")
	private String getApplicationToRun(final String[] args) {
		for (int i = 0; i < args.length; i++) {
			if (args[i].equals("-testApplication") && i < args.length - 1) //$NON-NLS-1$
				return args[i + 1];
		}
		final IProduct product = Platform.getProduct();
		if (product != null)
			return product.getApplication();
		return DEFAULT_APP_3_0;
	}

	@Post("null != $return")
	private Display getDisplay() {
		Display display;
		display = Display.getCurrent();
		if (display == null) {
			display = Display.getDefault();
		}
		return display;
	}

	@Override
	public void runTests() {
		// used to set workbench in automated mode - actually disabled
		// _testableObject.testingStarting();

		final ITestApplication app = getApplication();
		final ProgressMonitorDialog dialog = new ProgressMonitorDialog(null);
		final IRunnableWithProgress runnable = new IRunnableWithProgress() {
			public void run(final IProgressMonitor monitor) {
				PlatformUI.getWorkbench().addWorkbenchListener(
						new IWorkbenchListener() {
							@Override
							public boolean preShutdown(
									final IWorkbench workbench, boolean forced) {
								monitor.setCanceled(true);
								app.dispose();
								return true;
							}

							@Override
							public void postShutdown(final IWorkbench workbench) {
								// Nothing to be done
							}
						});

				app.run(Platform.getCommandLineArgs(), monitor);
			}
		};

		getDisplay().syncExec(new Runnable() {
			@Override
			public void run() {
				try {
					dialog.run(true, true, runnable);
				} catch (InvocationTargetException e) {
					getRTTAppUIPlugin().logError(
							e.getTargetException().getLocalizedMessage());
				} catch (InterruptedException e) {
					getRTTAppUIPlugin().logInfo(
							"Operation canceled, through user request");

				}
			}
		});

		// restores workbench state and closes workbench - disabled
		// _testableObject.testingFinished();
	}

	/**
	 * Retrieves the test application used.
	 * 
	 * @return Test application
	 */
	@Post("null != $return")
	protected abstract ITestApplication getApplication();
}