package de.fernunihagen.rtt.ui.app;

import static de.fernunihagen.rtt.ui.app.internal.RTTAppUIPlugin.getRTTAppUIPlugin;
import static de.fernunihagen.rtt.internal.ResourceUtil.getResourceUtil;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Arrays;
import java.util.ArrayList;
import java.util.List;

import org.contract4j5.contract.Contract;

import org.eclipse.core.runtime.CoreException;

import org.eclipse.debug.core.ILaunchConfiguration;

import org.eclipse.pde.ui.launcher.IPDELauncherConstants;

/**
 * Launch delegate for RTT-based launch configurations.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class RTTLaunchConfigurationDelegate extends
		AbstractRTTLaunchConfiguration {

	private void reportError(final String message, final Throwable e)
			throws CoreException {
		getRTTAppUIPlugin().reportError(message, e);
	}

	@Override
	public String[] getProgramArguments(final ILaunchConfiguration configuration)
			throws CoreException {
		final String[] args = super.getProgramArguments(configuration);
		final List<String> programArgs = new ArrayList<String>();

		programArgs.addAll(Arrays.asList(args));

		if (configuration
				.getAttribute(IPDELauncherConstants.USE_PRODUCT, false)) {
			String product = configuration.getAttribute(
					IPDELauncherConstants.PRODUCT, "");
			if (product.length() > 0) {
				programArgs.add("-application");
				programArgs.add(configuration.getAttribute(
						IPDELauncherConstants.APPLICATION, UIToolTestBench.ID));
			}
		}

		Writer writer = null;
		File testconfiguration = null;
		try {
			testconfiguration = File.createTempFile("rtt", ".cfg");
			testconfiguration.deleteOnExit();

			writer = new FileWriter(testconfiguration);
			writer.write(configuration.getAttribute(DESCRIPTION, ""));
		} catch (IOException e) {
			reportError("Failed to create test configuration file.", e);
		} finally {
			getResourceUtil().close(writer);
		}

		programArgs.add("-testconfiguration");
		programArgs.add(testconfiguration.getAbsolutePath());

		return programArgs.toArray(new String[programArgs.size()]);
	}
}
