package de.fernunihagen.rtt.ui;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jface.action.IAction;

/**
 * Action adapter interface, used in UI for building context sensitive menu's,
 * buttons or toolbar's.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface IActionAdapter {
	/**
	 * Tests if action item is applicable for given selection
	 * 
	 * @param selected
	 *            List of selected objects
	 * 
	 * @return true if applicable otherwise false
	 */
	@Pre("null != selected")
	boolean isApplicable(Object[] selected);

	/**
	 * Retrieves the action adapter identifier.
	 * 
	 * @return The action adapter identifier
	 */
	@Post("null != $return")
	String getID();

	/**
	 * Retrieves the name of action adapter.
	 * 
	 * @return Name of action adapter
	 */
	String getName();

	/**
	 * Target action which can be applied on a selection of applicable objects.
	 * 
	 * @param selected
	 *            List of selected objects
	 * 
	 * @return Target action
	 */
	@Pre("null != selected")
	@Post("null != $return")
	IAction getTarget(Object[] selected);
}
