package de.fernunihagen.rtt.ui.internal.action;

import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getRTTCoreUIPlugin;

import java.util.Map;

import org.apache.log4j.Logger;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;

import de.fernunihagen.rtt.ui.IActionAdapter;

/**
 * Abstract action adapter.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class AbstractActionAdapter implements IActionAdapter {
	private final static String PROP_ID = "component.name";
	private final static String PROP_NAME = "name";

	@Invar("null != $target")
	private String _id;
	@Invar("null != $target")
	private String _name;

	private static final class LoggerHolder {
		private static final Logger _DEFAULT = getRTTCoreUIPlugin().getLogger(
				AbstractActionAdapter.class.getName());
	}

	@Post("null != $return")
	private Logger getLogger() {
		return LoggerHolder._DEFAULT;
	}

	/**
	 * Creates an abstract action adapter.
	 */
	public AbstractActionAdapter() {
		_id = getClass().getName().toLowerCase();
		_name = getClass().getSimpleName().toLowerCase();
	}
	
	/**
	 * Component activation.
	 * 
	 * @param properties
	 *            Property map
	 */
	protected void activate(final Map<?, ?> properties) {
		_id = (String) properties.get(PROP_ID);

		if (properties.containsKey(PROP_NAME)) {
			_name = (String) properties.get(PROP_NAME);
		}

		if (getLogger().isDebugEnabled()) {
			final String message = String.format("[%s (%s) : activated",
					getID(), properties.toString());
			getLogger().debug(message);
		}
	}	

	@Override
	public String getID() {
		return _id;
	}

	@Override
	public String getName() {
		return _name;
	}
}
