package de.fernunihagen.rtt.ui.internal.jdt;

import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getImageDescriptor;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;

import de.fernunihagen.rtt.framework.ITestElement;

import de.fernunihagen.rtt.framework.model.ExceptionTestElement;

import de.fernunihagen.rtt.internal.jdt.oracle.JUnitTestCaseElement;

import de.fernunihagen.rtt.ui.internal.action.AbstractActionAdapter;

/**
 * Dump to console action adapter.
 * 
 * @author Osama El Hosami
 */
public class DumpToConsole extends AbstractActionAdapter {

	@Override
	public IAction getTarget(final Object[] selected) {
		ITestElement testElement = null;
		for (Object element : selected) {
			if (element instanceof JUnitTestCaseElement
					|| element instanceof ExceptionTestElement) {
				testElement = (ITestElement) element;
				break;
			}
		}

		if (null != testElement) {
			final ITestElement element = testElement;
			return new Action(getName(),
					getImageDescriptor("etool16/stackframe.gif")) {
				@Override
				public void run() {
					if (element instanceof JUnitTestCaseElement) {
						final JUnitTestCaseElement junitElement = (JUnitTestCaseElement) element;

						final String str = String.format(
								"RTT JUnit\n%s: %s\nTrace:\n%s",
								junitElement.getTestResult(), junitElement
										.getLabel(), junitElement
										.getStackFrame());

						System.out.println(str);

					} else if (element instanceof ExceptionTestElement) {
						final ExceptionTestElement exceptionElement = (ExceptionTestElement) element;

						final String str = String.format("RTT %s\n%s\n%s",
								exceptionElement.getTarget().getName(),
								exceptionElement.getLabel(), exceptionElement
										.getStackFrame());

						System.out.println(str);
					}
				}
			};
		}
		return null;
	}

	private boolean isFailedJUnitTestCaseElement(final Object selected) {
		return (selected instanceof JUnitTestCaseElement && !((JUnitTestCaseElement) selected)
				.getTestResult().isSuccess());
	}

	@Override
	public boolean isApplicable(final Object[] selected) {
		for (Object element : selected) {
			if (isFailedJUnitTestCaseElement(element)
					|| element instanceof ExceptionTestElement) {
				return true;
			}
		}
		return false;
	}
}
