package de.fernunihagen.rtt.ui.internal.jdt.launcher;

import java.util.ArrayList;

import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jdt.core.IImportContainer;
import org.eclipse.jdt.core.IImportDeclaration;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IPackageDeclaration;
import org.eclipse.jdt.core.IParent;

import org.eclipse.jdt.core.JavaModelException;

/**
 * Test subject.
 * 
 * @author Osama El Hosami
 */
public final class TestSubject extends AbstractTestSubject {

	private static final TestSubject[] EMPTY = new TestSubject[0];

	private final AbstractTestSubject _parent;

	/**
	 * Creates a test subject.
	 * 
	 * @param configuration
	 *            Test configuration
	 * @param element
	 *            Java element
	 * @param parent
	 *            Parent
	 */
	@Pre("null != configuration && null != element && null != parent")
	public TestSubject(final ToolTestConfiguration configuration,
			final IJavaElement element, final AbstractTestSubject parent) {
		super(configuration, element);
		_parent = parent;
	}

	private AbstractTestSubject[] getChildren(final IParent parent) {
		final ArrayList<AbstractTestSubject> elements = new ArrayList<AbstractTestSubject>();
		try {
			final IJavaElement[] children = parent.getChildren();
			elements.ensureCapacity(children.length);

			for (IJavaElement element : children) {
				if (!(element instanceof IImportContainer
						|| element instanceof IImportDeclaration || element instanceof IPackageDeclaration)) {
					final TestSubject testelement = new TestSubject(
							getConfiguration(), element, this);
					elements.add(testelement);
				}
			}
		} catch (JavaModelException e) {
		}
		elements.trimToSize();
		return elements.toArray(new AbstractTestSubject[elements.size()]);
	}

	@Post
	@Override
	public AbstractTestSubject[] getChildren() {
		if (getJavaElement() instanceof IParent) {
			return getChildren((IParent) getJavaElement());
		}
		return EMPTY;
	}

	@Override
	public AbstractTestSubject getParent() {
		return _parent;
	}
}
