package de.fernunihagen.rtt.ui.internal.jdt.launcher;

import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getRTTCoreUIPlugin;

import java.io.ByteArrayInputStream;
import java.io.InputStreamReader;
import java.io.StringReader;
import java.io.StringWriter;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.SubMonitor;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaModel;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

import org.eclipse.pde.core.plugin.IFragmentModel;
import org.eclipse.pde.core.plugin.IPluginModelBase;
import org.eclipse.pde.core.plugin.PluginRegistry;

import org.eclipse.swt.custom.BusyIndicator;

import org.eclipse.swt.widgets.Display;

import de.fernunihagen.rtt.framework.ElementFilterOp;
import de.fernunihagen.rtt.framework.IElementCollector;
import de.fernunihagen.rtt.framework.IElementFilter;
import de.fernunihagen.rtt.framework.ITestActor;
import de.fernunihagen.rtt.framework.ITestActorProvider;
import de.fernunihagen.rtt.framework.ITestConfiguration;
import de.fernunihagen.rtt.framework.ITestDescription;
import de.fernunihagen.rtt.framework.RTTCore;
import de.fernunihagen.rtt.framework.XMLTestConfiguration;

import de.fernunihagen.rtt.framework.repository.IDiff;
import de.fernunihagen.rtt.framework.repository.IDiffVisitor;
import de.fernunihagen.rtt.framework.repository.IRepository;
import de.fernunihagen.rtt.framework.repository.IRepositoryProvider;

import de.fernunihagen.rtt.framework.testadapter.ReflectiveRefactoringTestAdapter;
import de.fernunihagen.rtt.framework.util.IOverwriteQuery;

import de.fernunihagen.rtt.internal.jdt.ExcludedJavaElementFilter;

import de.fernunihagen.rtt.internal.util.MutableBoolean;
import de.fernunihagen.rtt.internal.util.Subject;

import de.fernunihagen.rtt.ui.internal.jdt.ToolTestCollector;

/**
 * Tool test configuration.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class ToolTestConfiguration
		extends
		Subject<ToolTestConfiguration, IToolTestConfigurationChangeListener, ToolTestConfigurationChangeEvent> {
	@Invar("null != $target")
	private final Map<IProject, String> _inSync;
	@Invar("null != $target")
	private final Map<IProject, String> _outOfSync;
	@Invar("null != $target")
	private final List<TestSubjectContainer> _projects;
	@Invar("null != $target")
	private final XMLTestConfiguration _configuration;
	@Invar("null != $target")
	private final Map<ITestActor, ExcludedJavaElementFilter> _actorExcludeFilter;
	@Invar("null != $target")
	private final Map<ActorBundleId, Set<String>> _actorBundleTestMembers;
	@Invar("null != $target")
	private IElementCollector<? extends IAdaptable> _testCollector;
	@Invar("null != $target")
	private final Set<String> _selectedTestMemberIds;

	private static final String TESTSUBJECT_CONFIGFILE = ".rtt.config";

	private static final class ActorBundleId {
		@Invar("null != $target")
		public final ITestActor actor;
		@Invar("null != $target")
		public final String bundleId;

		public ActorBundleId(final ITestActor actor, final String bundleId) {
			this.actor = actor;
			this.bundleId = bundleId;
		}

		@Override
		public int hashCode() {
			return actor.hashCode() + bundleId.hashCode();
		}

		@Override
		public boolean equals(final Object other) {
			if (null == other || !(other instanceof ActorBundleId)) {
				return false;
			}
			if (this == other) {
				return true;
			}
			final ActorBundleId otherActorBundleId = (ActorBundleId) other;
			return actor.equals(otherActorBundleId.actor)
					&& bundleId.equals(otherActorBundleId.bundleId);
		}
	}

	/**
	 * Creates a tool test configuration.
	 * 
	 * @param configuration
	 *            Configuration
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != configuration")
	public ToolTestConfiguration(final String configuration)
			throws CoreException {
		_configuration = XMLTestConfiguration.createReadRoot(new StringReader(
				configuration));
		_inSync = new HashMap<IProject, String>();
		_outOfSync = new HashMap<IProject, String>();
		_actorExcludeFilter = new HashMap<ITestActor, ExcludedJavaElementFilter>();
		_actorBundleTestMembers = new HashMap<ActorBundleId, Set<String>>();
		_selectedTestMemberIds = new HashSet<String>();
		_testCollector = IElementCollector.NULL;
		_projects = new ArrayList<TestSubjectContainer>();
	}

	/**
	 * Initializes configuration.
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	public void initialize(final IProgressMonitor pm) throws CoreException {
		fillActorConfigurationCache();
		importProjects(_configuration.getChild(ITestDescription.TYPE_PROJECTS),
				pm);
	}

	/**
	 * Retrieves the bundle symbolic name of a project, returns null if project
	 * is not a plug-in project.
	 * 
	 * @param project
	 *            Project
	 * 
	 * @return Bundle symbolic name
	 */
	public static String getBundleId(final IJavaProject project) {
		if (null == project) {
			return null;
		}
		final IPluginModelBase model = PluginRegistry.findModel(project
				.getProject());
		if (null == model) {
			return null;
		}
		if (model instanceof IFragmentModel) {
			return ((IFragmentModel) model).getFragment().getPluginId();
		}
		return model.getPluginBase().getId();
	}

	@Pre("null != testActor && null != bundles")
	private void fillActorBundleConfigurationCache(final ITestActor testActor,
			final ITestConfiguration[] bundles) {
		for (ITestConfiguration bundle : bundles) {
			final ITestConfiguration[] testMembers = bundle
					.getChildren(ITestDescription.TYPE_ACTORS_ACTOR_TESTBUNDLE_TESTCASE);

			for (ITestConfiguration testMember : testMembers) {
				final String testMemberId = testMember.get(
						ITestDescription.KEY_NAME, (String) null);
				if (null != testMemberId) {
					_selectedTestMemberIds.add(testMemberId);
				}
			}

			final String bundleId = bundle.get(ITestDescription.KEY_NAME,
					(String) null);
			if (null != bundleId) {
				_actorBundleTestMembers.put(new ActorBundleId(testActor,
						bundleId), new HashSet<String>());
			}
		}
	}

	@Pre("null != testActor && null != bundles")
	private void fillActorExcludedConfigurationCache(
			final ITestActor testActor, final ITestConfiguration[] excluded) {
		final Set<String> excludedElementIdentifer = new HashSet<String>(
				excluded.length);

		for (ITestConfiguration exclude : excluded) {
			final String handleIdentifier = exclude.get(
					ITestDescription.KEY_NAME, (String) null);
			if (null != handleIdentifier) {
				excludedElementIdentifer.add(handleIdentifier);
			}
		}

		final String[] identifiers = excludedElementIdentifer
				.toArray(new String[excludedElementIdentifer.size()]);

		ExcludedJavaElementFilter filter = _actorExcludeFilter.get(testActor);
		if (null == filter) {
			filter = new ExcludedJavaElementFilter();
		}

		filter.addAll(identifiers);
		_actorExcludeFilter.put(testActor, filter);
	}

	@Pre("null != actors && null != provider")
	private void fillActorConfigurationCache(final ITestConfiguration[] actors,
			final ITestActorProvider<?> provider) {
		for (ITestConfiguration actor : actors) {
			final String id = actor.get(ITestDescription.KEY_NAME,
					(String) null);
			if (null != id) {
				try {
					final ITestActor testActor = provider.get(id);
					ITestConfiguration[] configuration;
					if (testActor instanceof ReflectiveRefactoringTestAdapter) {
						configuration = actor
								.getChildren(ITestDescription.TYPE_ACTORS_ACTOR_TESTBUNDLE);
						fillActorBundleConfigurationCache(testActor,
								configuration);
					}
					configuration = actor
							.getChildren(ITestDescription.TYPE_ACTORS_ACTOR_EXCLUDE);
					fillActorExcludedConfigurationCache(testActor,
							configuration);

				} catch (CoreException e) {
				}
			}
		}
	}

	private void fillActorConfigurationCache() {
		fillActorConfigurationCache(_configuration);
	}

	@Pre("null != configuration")
	private void fillActorConfigurationCache(
			final ITestConfiguration configuration) {
		final ITestConfiguration actors = configuration
				.getChild(ITestDescription.TYPE_ACTORS);
		if (null == actors) {
			return;
		}

		ITestConfiguration[] oracles = actors
				.getChildren(ITestDescription.TYPE_ACTORS_ORACLE);
		try {
			fillActorConfigurationCache(oracles, RTTCore.getOracleProvider());
		} catch (CoreException e) {
		}

		ITestConfiguration[] testadapters = actors
				.getChildren(ITestDescription.TYPE_ACTORS_TESTADAPTER);
		try {
			fillActorConfigurationCache(testadapters,
					RTTCore.getRefactoringTestAdapterProvider());
		} catch (CoreException e) {
		}
	}

	/**
	 * Tests if element is excluded in actor configuration.
	 * 
	 * @param actor
	 *            Test actor
	 * @param element
	 *            Tool test element
	 * 
	 * @return True if satisfied otherwise false
	 */
	public boolean isExcluded(final ITestActor actor,
			final AbstractTestSubject element) {
		if (null == actor || null == element) {
			return true;
		}
		final ExcludedJavaElementFilter filter = _actorExcludeFilter.get(actor);
		return null != filter && !filter.accept(element.getJavaElement());
	}

	/**
	 * Retrieves the list of configured test actors
	 * 
	 * @return Test actors
	 */
	@Post("null != $return")
	public ITestActor[] getTestActors() {
		ITestActor[] actors = _actorExcludeFilter.keySet().toArray(
				new ITestActor[0]);
		Arrays.sort(actors, new Comparator<ITestActor>() {
			@Override
			@Pre("null != a1 && null != a2")
			public int compare(final ITestActor a1, final ITestActor a2) {
				return a2.getName().compareTo(a1.getName());
			}
		});
		return actors;
	}

	/**
	 * Retrieves the hole test configuration.
	 * 
	 * @return Test configuration
	 */
	@Post("null != $return")
	public ITestConfiguration getConfiguration() {
		return _configuration;
	}

	@Post("null != $return")
	private Display getDisplay() {
		Display display;
		display = Display.getCurrent();
		if (display == null) {
			display = Display.getDefault();
		}
		return display;
	}

	/**
	 * Retrieves the list of available test subject container.
	 * 
	 * @return Test subject container
	 */
	@Post("null != $return")
	public TestSubjectContainer[] getTestSubjectContainer() {
		if (_projects.isEmpty()) {
			final IProject[] projects = getProjects();

			BusyIndicator.showWhile(getDisplay(), new Runnable() {
				@Override
				public void run() {
					final ITestActor[] actors = getTestActors();
					for (IProject project : projects) {
						try {
							if (project.isAccessible()
									&& project.hasNature(JavaCore.NATURE_ID)) {
								final TestSubjectContainer element = new TestSubjectContainer(
										ToolTestConfiguration.this, project);

								element.initialize(actors, null);
								_projects.add(element);
							}
						} catch (CoreException e) {
						}
					}
				}
			});
		}

		return _projects.toArray(new TestSubjectContainer[_projects.size()]);
	}

	/**
	 * Retrieves the list of imported projects.
	 * 
	 * @return Projects
	 */
	@Post("null != $return")
	public IProject[] getProjects() {
		final List<IProject> projects = new ArrayList<IProject>(_inSync.size()
				+ _outOfSync.size());

		projects.addAll(_outOfSync.keySet());
		projects.addAll(_inSync.keySet());

		return projects.toArray(new IProject[projects.size()]);
	}

	/**
	 * Retrieves the excluded elements filter.
	 * 
	 * @param actor
	 *            Test actor
	 * 
	 * @return Excluded element filter or null
	 */
	@Pre("null != actor")
	public IElementFilter<IAdaptable> getExcludedElementFilter(
			final ITestActor actor) {
		return _actorExcludeFilter.get(actor);
	}

	/**
	 * Retrieves the test element filter.
	 * 
	 * @return Test element filter
	 */
	@Post("null != $return")
	public IElementFilter<IAdaptable> getTestElementFilter() {
		return ElementFilterOp.adaptToElementFilter(_testCollector);
	}

	/**
	 * Excludes element from actor configuration.
	 * 
	 * @param actor
	 *            Test actor
	 * @param element
	 *            Element
	 * @param fState
	 *            State
	 */
	@Pre("null != actor && null != element")
	public void exclude(final ITestActor actor,
			final AbstractTestSubject element, boolean fState) {
		final ExcludedJavaElementFilter filter = _actorExcludeFilter.get(actor);
		if (null != filter) {
			final IJavaElement javaElement = element.getJavaElement();
			if (fState) {
				filter.add(javaElement);
			} else {
				filter.remove(javaElement);
			}
		}
	}

	private IStatus createError(final String message, final Throwable e) {
		return getRTTCoreUIPlugin().createError(message, e);
	}

	private void reportError(final CoreException[] exceptions,
			final String message) throws CoreException {
		getRTTCoreUIPlugin().reportError(exceptions, message);
	}

	private void reportError(final IStatus[] statuses, final String message)
			throws CoreException {
		getRTTCoreUIPlugin().reportError(statuses, message);
	}

	private void reportError(final String message, final Throwable e)
			throws CoreException {
		getRTTCoreUIPlugin().reportError(message, e);
	}

	@Pre("null != repositoryProvider && null != configuration && null != query && null != imported")
	private void importProjects(final IRepositoryProvider repositoryProvider,
			final ITestConfiguration configuration,
			final IOverwriteQuery query, final Map<IProject, String> imported,
			final IProgressMonitor pm) throws CoreException {
		try {
			final SubMonitor monitor = SubMonitor.convert(pm,
					"Importing projects", SubMonitor.UNKNOWN);

			final List<CoreException> errors = new ArrayList<CoreException>();
			final Set<String> referenceStrings = new HashSet<String>();
			final ITestConfiguration[] providers = configuration
					.getChildren(ITestDescription.TYPE_PROJECTS_PROVIDER);

			for (ITestConfiguration provider : providers) {
				final ITestConfiguration[] projects = provider
						.getChildren(ITestDescription.TYPE_PROJECTS_PROVIDER_PROJECT);

				for (ITestConfiguration project : projects) {
					referenceStrings.add(project.get(ITestDescription.KEY_NAME,
							(String) null));
				}

				try {
					final String id = provider.get(ITestDescription.KEY_NAME,
							(String) null);

					final IRepository repository = repositoryProvider
							.getRepository(id);

					final String[] references = referenceStrings
							.toArray(new String[referenceStrings.size()]);

					final IProject[] allProjects = repository.checkout(
							references, query, monitor.newChild(1));

					for (int i = 0; i < allProjects.length; i++) {
						imported.put(allProjects[i], references[i]);
					}

					referenceStrings.clear();
				} catch (CoreException e) {
					errors.add(e);
				}
			}

			final Iterator<Map.Entry<IProject, String>> ita = imported
					.entrySet().iterator();
			while (ita.hasNext()) {
				final Map.Entry<IProject, String> project = ita.next();
				if (!project.getKey().exists()) {
					ita.remove();

					errors.add(new CoreException(createError(
							"Failed to checkout " + project.getKey().getName(),
							null)));
				}
			}

			if (!errors.isEmpty()) {
				if (errors.size() == 1) {
					throw errors.get(0);
				} else {
					reportError(
							errors.toArray(new CoreException[errors.size()]),
							"The following errors occurred while importing projects. Some projects may not be loaded.");
				}
			}

		} finally {
			if (null != pm) {
				pm.done();
			}
		}
	}

	@Post("null != $return")
	private IWorkspace getWorkspace() {
		return getRTTCoreUIPlugin().getWorkspace();
	}

	/**
	 * Adds a project to configuration and creates corresponding test subject
	 * container.
	 * 
	 * @param project
	 *            Project
	 * @param pm
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @return Test subject container
	 */
	@Pre("null != project")
	public TestSubjectContainer addProject(final IProject project,
			final IProgressMonitor pm) {
		final TestSubjectContainer element;

		try {
			SubMonitor monitor = SubMonitor.convert(pm, "Adding project", 2);

			element = new TestSubjectContainer(ToolTestConfiguration.this,
					project);

			loadProjectConfiguration(element);
			element.initialize(getTestActors(), monitor.newChild(1));
			_projects.add(element);

			setChanged();
			notifyObservers();
			monitor.worked(1);

		} finally {
			if (null != pm) {
				pm.done();
			}
		}

		return element;
	}

	@Pre("null != project")
	private boolean addProject(final ITestConfiguration configuration,
			final IProject project) throws CoreException {
		if (null == configuration) {
			return false;
		}

		final IRepositoryProvider provider = RTTCore.getRepositoryProvider();
		final IRepository repository = provider.getRepository(project);
		final String reference = addProjectReference(provider, configuration,
				project, null);

		final ITestConfiguration[] providerConfigurations = configuration
				.getChildren(ITestDescription.TYPE_PROJECTS_PROVIDER);
		ITestConfiguration providerConfiguration = null;

		for (ITestConfiguration providerConfigurationNode : providerConfigurations) {
			final String id = providerConfigurationNode.get(
					ITestDescription.KEY_NAME, "");
			if (id.equals(repository.getID())) {
				providerConfiguration = providerConfigurationNode;
				break;
			}
		}

		if (null == providerConfiguration) {
			providerConfiguration = configuration
					.createChild(ITestDescription.TYPE_PROJECTS_PROVIDER);
			providerConfiguration.put(ITestDescription.KEY_NAME,
					repository.getID());
		}

		providerConfiguration.createChild(
				ITestDescription.TYPE_PROJECTS_PROVIDER_PROJECT).put(
				ITestDescription.KEY_NAME, reference);

		return true;
	}

	@Pre("null != provider && null != configuration && null != project")
	private String addProjectReference(final IRepositoryProvider provider,
			final ITestConfiguration configuration, final IProject project,
			final IProgressMonitor pm) throws CoreException {
		final SubMonitor monitor = SubMonitor.convert(pm);
		// test existing projects has not been modified and has the
		// requested revision
		final MutableBoolean fIsModified = new MutableBoolean(false);
		final IDiffVisitor visitor = new IDiffVisitor() {
			@Override
			public boolean visit(final IDiff diff) {
				fIsModified.setValue(true);
				return false;
			}
		};

		project.open(monitor.newChild(1));

		final IRepository repository = provider.getRepository(project);
		final String referenceString = repository.getReferences(
				new IProject[] { project }, null)[0];

		final IResource[] resources = new IResource[] { project };

		try {
			repository.refreshLocal(resources, IResource.DEPTH_INFINITE,
					monitor.newChild(1));
			repository.getSubscriber().accept(resources,
					IResource.DEPTH_INFINITE, visitor);

			if (!fIsModified.getValue()) {
				_inSync.put(project, referenceString);
			} else {
				_outOfSync.put(project, referenceString);
			}
		} catch (CoreException e) {
			_outOfSync.put(project, referenceString);
		}

		return referenceString;
	}

	private void importProjects(final ITestConfiguration configuration,
			final IProgressMonitor pm) throws CoreException {
		if (null == configuration) {
			return;
		}

		_inSync.clear();
		_outOfSync.clear();

		final SubMonitor monitor = SubMonitor.convert(pm);
		final List<String> existing = new ArrayList<String>();
		final IRepositoryProvider provider = RTTCore.getRepositoryProvider();
		final List<IStatus> errors = new ArrayList<IStatus>();

		try {
			importProjects(provider, configuration, new IOverwriteQuery() {
				@Override
				public StatusCode queryOverwrite(String pathString) {
					existing.add(pathString);
					return StatusCode.NO;
				}
			}, _inSync, monitor.newChild(1));
		} catch (CoreException e) {
			errors.add(e.getStatus());
		}

		for (String projectName : existing) {
			if (monitor.isCanceled()) {
				throw new OperationCanceledException();
			}
			try {
				final IProject project = getWorkspace().getRoot().getProject(
						Path.fromOSString(projectName).lastSegment());

				addProjectReference(provider, configuration, project,
						monitor.newChild(1));
			} catch (CoreException e) {
				errors.add(e.getStatus());
			}
		}

		if (!errors.isEmpty()) {
			if (errors.size() == 1) {
				throw new CoreException(errors.get(0));
			} else {
				reportError(
						errors.toArray(new IStatus[errors.size()]),
						"The following errors occurred while importing projects. Some projects may not be available.");
			}
		}
	}

	/**
	 * Tests if resource is in sync with repository.
	 * 
	 * @param resource
	 *            Resource
	 * 
	 * @return True if satisfied otherwise false
	 */
	public boolean isInSync(final IResource resource) {
		if (null == resource || !resource.isAccessible()) {
			return false;
		}

		final IProject project = resource.getProject();
		if (_inSync.containsKey(project)) {
			return true;
		}

		try {
			final IRepository repository = RTTCore.getRepositoryProvider()
					.getRepository(project);

			if (!repository.getSubscriber().isSupervised(resource)) {
				return false;
			}

			return repository.getSubscriber().getDiff(resource).getKind() == IDiff.NO_CHANGE;
		} catch (CoreException e) {
		}

		return false;
	}

	/**
	 * Removes all projects from configuration.
	 */
	public void removeAllProjects() {
		final Set<Map.Entry<ITestActor, ExcludedJavaElementFilter>> excludedEntrySet = _actorExcludeFilter
				.entrySet();
		for (Map.Entry<ITestActor, ExcludedJavaElementFilter> entry : excludedEntrySet) {
			entry.getValue().clear();
		}
		_inSync.clear();
		_outOfSync.clear();
		_projects.clear();

		setChanged();
		notifyObservers();
	}

	@Pre("null != actor && null != container")
	private void loadProjectConfigurations(final ITestActor actor,
			final TestSubjectContainer container) {
		final ITestConfiguration configuration = getProjectConfiguration(container);
		if (null == configuration) {
			return;
		}

		final ITestConfiguration actors = configuration
				.getChild(ITestDescription.TYPE_ACTORS);
		if (null == actors) {
			return;
		}

		final ITestConfiguration[] candidates = actors.getChildren(actor
				.getTypeName());
		for (ITestConfiguration candidate : candidates) {
			final String id = candidate.get(ITestDescription.KEY_NAME, "");
			if (!id.equals(actor.getID())) {
				continue;
			}

			ITestConfiguration[] excluded = candidate
					.getChildren(ITestDescription.TYPE_ACTORS_ACTOR_EXCLUDE);
			fillActorExcludedConfigurationCache(actor, excluded);
			break;
		}
	}

	@Pre("null != actor")
	private void loadProjectConfigurations(final ITestActor actor) {
		final TestSubjectContainer[] containers = getTestSubjectContainer();
		for (TestSubjectContainer container : containers) {
			loadProjectConfigurations(actor, container);
		}
	}

	@Pre("null != element")
	private ITestConfiguration getProjectConfiguration(
			final TestSubjectContainer element) {
		final IJavaProject javaProject = element.getJavaElement();
		final IProject project = javaProject.getProject();
		final IFile file = project.getFile(TESTSUBJECT_CONFIGFILE);
		if (!file.exists()) {
			return null;
		}

		ITestConfiguration configuration;
		try {
			configuration = XMLTestConfiguration
					.createReadRoot(new InputStreamReader(file
							.getContents(true)));
		} catch (CoreException e) {
			configuration = null;
		}

		return configuration;
	}

	/**
	 * Loads project configuration.
	 * 
	 * @param element
	 *            project
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != element")
	public void loadProjectConfiguration(final TestSubjectContainer element) {
		final ITestActor[] actors = getTestActors();
		for (ITestActor actor : actors) {
			loadProjectConfigurations(actor, element);
		}
	}

	/**
	 * Saves project configuration.
	 * 
	 * @param element
	 *            project
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != element")
	public void saveProjectConfiguration(final TestSubjectContainer element)
			throws CoreException {

		final StringWriter writer = new StringWriter();
		final IJavaProject javaProject = element.getJavaElement();
		final IProject project = javaProject.getProject();
		final IFile file = project.getFile(TESTSUBJECT_CONFIGFILE);
		final XMLTestConfiguration configuration = XMLTestConfiguration
				.createWriteRoot("configuration");

		final ITestConfiguration actors = configuration
				.createChild(ITestDescription.TYPE_ACTORS);

		final Set<Map.Entry<ITestActor, ExcludedJavaElementFilter>> excludedEntrySet = _actorExcludeFilter
				.entrySet();
		for (Map.Entry<ITestActor, ExcludedJavaElementFilter> entry : excludedEntrySet) {
			final ITestConfiguration actor = actors.createChild(entry.getKey()
					.getTypeName());
			actor.put(ITestDescription.KEY_NAME, entry.getKey().getID());

			final String[] identifiers = entry.getValue().getHandleIdentifiers(
					javaProject);

			for (String identifier : identifiers) {
				final ITestConfiguration exclude = actor
						.createChild(ITestDescription.TYPE_ACTORS_ACTOR_EXCLUDE);
				exclude.put(ITestDescription.KEY_NAME, identifier);
			}
		}

		configuration.save(writer);

		if (file.exists()) {
			file.delete(true, false, null);
		}

		file.create(new ByteArrayInputStream(writer.toString().getBytes()),
				true, null);
	}

	/**
	 * Removes a project from configuration.
	 * 
	 * @param element
	 *            project
	 * 
	 * @return true if succeeded otherwise false
	 */
	@Pre("null != element")
	public boolean removeProject(final TestSubjectContainer element) {
		final IProject project = element.getJavaElement().getProject();

		// remove the excluded configuration of project from all test actors
		for (ITestActor actor : getTestActors()) {
			element.exclude(actor, false);
		}

		_inSync.remove(project);
		_outOfSync.remove(project);
		_projects.remove(element);

		setChanged();
		notifyObservers();

		return true;
	}

	/**
	 * Removes an actor from configuration.
	 * 
	 * @param actor
	 *            Test actor
	 * 
	 * @return True if succeeded otherwise false
	 */
	@Pre("null != actor")
	public boolean removeActor(final ITestActor actor) {
		final Iterator<ActorBundleId> iterator = _actorBundleTestMembers
				.keySet().iterator();
		while (iterator.hasNext()) {
			ActorBundleId id = iterator.next();
			if (id.actor.equals(actor)) {
				iterator.remove();
				break;
			}
		}

		_actorExcludeFilter.remove(actor);

		setChanged();
		notifyObservers();

		return true;
	}

	/**
	 * Adds an test actor to configuration
	 * 
	 * @param actor
	 *            Test actor
	 */
	@Pre("null != actor")
	public void addActor(final ITestActor actor) {
		if (_actorExcludeFilter.containsKey(actor)) {
			throw new IllegalStateException();
		}

		_actorExcludeFilter.put(actor, new ExcludedJavaElementFilter());
		loadProjectConfigurations(actor);

		setChanged();
		notifyObservers();
	}

	/**
	 * Flushes the configuration cache
	 */
	public void flush() {
		final XMLTestConfiguration projects = _configuration
				.getChild(ITestDescription.TYPE_PROJECTS);
		if (null != projects) {
			projects.removeChildren(ITestDescription.TYPE_PROJECTS_PROVIDER);

			for (TestSubjectContainer element : _projects) {
				try {
					addProject(projects, element.getJavaElement().getProject());
				} catch (CoreException e) {
					e.printStackTrace();
				}
			}
		}

		final XMLTestConfiguration actors = _configuration
				.getChild(ITestDescription.TYPE_ACTORS);
		if (null != actors) {
			actors.removeChildren(ITestDescription.TYPE_ACTORS_ORACLE);
			actors.removeChildren(ITestDescription.TYPE_ACTORS_TESTADAPTER);

			final Map<String, ITestConfiguration> actorConfigurations = new HashMap<String, ITestConfiguration>();
			final Set<Map.Entry<ITestActor, ExcludedJavaElementFilter>> excludedEntrySet = _actorExcludeFilter
					.entrySet();
			for (Map.Entry<ITestActor, ExcludedJavaElementFilter> entry : excludedEntrySet) {
				final ITestConfiguration configuration = actors
						.createChild(entry.getKey().getTypeName());
				configuration.put(ITestDescription.KEY_NAME, entry.getKey()
						.getID());

				actorConfigurations.put(entry.getKey().getID(), configuration);

				final String[] identifiers = entry.getValue()
						.getHandleIdentifiers();

				for (String identifier : identifiers) {
					final ITestConfiguration exclude = configuration
							.createChild(ITestDescription.TYPE_ACTORS_ACTOR_EXCLUDE);
					exclude.put(ITestDescription.KEY_NAME, identifier);
				}
			}

			final Set<Map.Entry<ActorBundleId, Set<String>>> bundleEntrySet = _actorBundleTestMembers
					.entrySet();
			for (Map.Entry<ActorBundleId, Set<String>> entry : bundleEntrySet) {
				final ITestConfiguration configuration = actorConfigurations
						.get(entry.getKey().actor.getID());
				if (null == configuration) {
					continue;
				}

				final Set<String> identifiers = entry.getValue();

				final ITestConfiguration bundle = configuration
						.createChild(ITestDescription.TYPE_ACTORS_ACTOR_TESTBUNDLE);
				bundle.put(ITestDescription.KEY_NAME, entry.getKey().bundleId);

				for (String identifier : identifiers) {
					if (_selectedTestMemberIds.contains(identifier)) {
						final ITestConfiguration testMember = bundle
								.createChild(ITestDescription.TYPE_ACTORS_ACTOR_TESTBUNDLE_TESTCASE);
						testMember.put(ITestDescription.KEY_NAME, identifier);
					}
				}
			}
		}
	}

	@Override
	public String toString() {
		flush();
		final StringWriter writer = new StringWriter();
		_configuration.save(writer);
		String value = writer.toString();
		return value;
	}

	@Override
	protected ToolTestConfiguration getThis() {
		return this;
	}

	/**
	 * Retrieves the test bundle plug-in project
	 * 
	 * @param actor
	 *            Test actor
	 * 
	 * @return Test bundle plug-in project or null
	 */
	public IJavaProject getTestBundleProject(final ITestActor actor) {
		final String id = getTestBundleId(actor);
		if (null == id) {
			return null;
		}

		final IJavaModel model = JavaCore.create(getWorkspace().getRoot());
		if (null == model) {
			return null;
		}

		try {
			for (IJavaProject project : model.getJavaProjects()) {
				project.open(null);

				final String bundleId = getBundleId(project);
				if (null != bundleId && id.equals(bundleId)) {
					return project;
				}
			}
		} catch (JavaModelException e) {
		}

		return null;
	}

	/**
	 * Retrieves the first test bundle identifier of given actor.
	 * 
	 * @param actor
	 *            Test actor
	 * 
	 * @return Bundle identifier or null
	 */
	public String getTestBundleId(final ITestActor actor) {
		if (null == actor) {
			return null;
		}

		final Set<Map.Entry<ActorBundleId, Set<String>>> entrySet = _actorBundleTestMembers
				.entrySet();
		for (Map.Entry<ActorBundleId, Set<String>> entry : entrySet) {
			if (entry.getKey().actor == actor) {
				return entry.getKey().bundleId;
			}
		}

		return null;
	}

	/**
	 * Sets the test bundle configuration for given actor.
	 * 
	 * @param actor
	 *            Test actor
	 * @param javaProject
	 *            Plug-in project
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>
	 *             If an error occurs</li>
	 *             </ul>
	 */
	public void setTestBundle(final ITestActor actor,
			final IJavaProject javaProject) throws CoreException {
		final String bundleId = getBundleId(javaProject);

		if (null == actor
				|| !(actor instanceof ReflectiveRefactoringTestAdapter)
				|| null == bundleId) {
			_actorBundleTestMembers.clear();
			_testCollector.clear();
			return;
		}

		_testCollector = new ToolTestCollector(
				new IProject[] { javaProject.getProject() });
		final CoreException[] exceptionRef = { null };

		BusyIndicator.showWhile(getDisplay(), new Runnable() {
			@Override
			public void run() {
				try {
					_testCollector.collect(null);
				} catch (CoreException e) {
					exceptionRef[0] = e;
				}
			}
		});

		final ActorBundleId actorBundleId = new ActorBundleId(actor, bundleId);
		final List<? extends IAdaptable> testMembers = _testCollector
				.getMatches();
		final Set<String> testMemberIds = new HashSet<String>(
				testMembers.size());
		for (IAdaptable testMember : testMembers) {
			if (testMember instanceof IType) {
				testMemberIds.add(((IType) testMember).getFullyQualifiedName());
			}
		}

		if (!_actorBundleTestMembers.containsKey(actorBundleId)) {
			_selectedTestMemberIds.addAll(testMemberIds);
		}

		// FIXME: actually implementation supports only one test bundle per
		// actor configuration
		_actorBundleTestMembers.clear();

		_actorBundleTestMembers.put(actorBundleId, testMemberIds);

		setChanged();
		notifyObservers();

		if (null != exceptionRef[0]) {
			reportError(null, exceptionRef[0]);
		}
	}

	/**
	 * Retrieves the list of selected test members.
	 * 
	 * @return List of test members
	 */
	@Post("null != $return")
	public IAdaptable[] getTestMembers() {
		final List<? extends IAdaptable> matches = _testCollector.getMatches();
		final List<IAdaptable> result = new ArrayList<IAdaptable>(
				matches.size());
		final Map<String, IAdaptable> testMembers = new HashMap<String, IAdaptable>(
				matches.size());

		for (IAdaptable testMember : matches) {
			if (testMember instanceof IType) {
				testMembers.put(((IType) testMember).getFullyQualifiedName(),
						testMember);
			}
		}

		final Iterator<String> testMemberIdIta = _selectedTestMemberIds
				.iterator();
		while (testMemberIdIta.hasNext()) {
			final String id = testMemberIdIta.next();
			final IAdaptable testMember = testMembers.get(id);
			if (null != testMember) {
				result.add(testMember);
			} else {
				testMemberIdIta.remove();
			}
		}

		return result.toArray(new IAdaptable[result.size()]);
	}

	/**
	 * Retrieves the list of available test members.
	 * 
	 * @return List of test members
	 */
	@Post("null != $return")
	public IAdaptable[] getAllTestMembers() {
		final List<? extends IAdaptable> members = _testCollector.getMatches();
		return members.toArray(new IAdaptable[members.size()]);
	}

	/**
	 * Tests if element is contained in set of test members.
	 * 
	 * @param element
	 *            Test member
	 * 
	 * @return True if satisfied otherwise false
	 */
	boolean containsTestMember(final Object element) {
		if (null == element || !(element instanceof IType)) {
			return false;
		}

		return _selectedTestMemberIds.contains(((IType) element)
				.getFullyQualifiedName());
	}

	/**
	 * Retrieves the first test adapter in test configuration.
	 * 
	 * @param id
	 *            Test actor identifier
	 * 
	 * @return Test adapter or null
	 */
	ITestActor getTestAdapter(final String id) {
		final ITestActor[] actors = getTestActors();
		for (ITestActor actor : actors) {
			if (actor.getID().equals(id)) {
				return actor;
			}
		}

		return null;
	}

	/**
	 * Adds a test member
	 * 
	 * @param element
	 *            Test member
	 */
	void removeTestMember(final IAdaptable element) {
		if (null == element || !(element instanceof IType)) {
			return;
		}

		_selectedTestMemberIds
				.remove(((IType) element).getFullyQualifiedName());
		setChanged();
		notifyObservers();
	}

	/**
	 * Adds a test member
	 * 
	 * @param element
	 *            Test member
	 */
	void addTestMember(final IAdaptable element) {
		if (null == element || !(element instanceof IType)) {
			return;
		}

		_selectedTestMemberIds.add(((IType) element).getFullyQualifiedName());
		setChanged();
		notifyObservers();
	}

	/**
	 * Adds a set of test members to configuration
	 * 
	 * @param testMembers
	 *            List of test members
	 */
	@Pre("null != testMembers")
	void addAllTestMember(final IAdaptable[] testMembers) {
		for (IAdaptable testMember : testMembers) {
			if (testMember instanceof IType) {
				_selectedTestMemberIds.add(((IType) testMember)
						.getFullyQualifiedName());
			}
		}

		setChanged();
		notifyObservers();
	}

	/**
	 * Removes a set of test members from configuration
	 * 
	 * @param testMembers
	 *            List of test members
	 */
	@Pre("null != testMembers")
	void removeAllTestMember(final IAdaptable[] testMembers) {
		for (IAdaptable testMember : testMembers) {
			if (testMember instanceof IType) {
				_selectedTestMemberIds.remove(((IType) testMember)
						.getFullyQualifiedName());
			}
		}

		setChanged();
		notifyObservers();
	}
}
