package de.fernunihagen.rtt.ui.internal.jdt.launcher;

import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getRTTCoreUIPlugin;

import java.io.StringWriter;

import java.lang.reflect.InvocationTargetException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Arrays;
import java.util.Set;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.core.runtime.jobs.Job;

import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;

import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;

import org.eclipse.jdt.ui.JavaElementLabelProvider;

import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.eclipse.ui.progress.UIJob;

import org.eclipse.jface.action.Action;

import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;

import org.eclipse.jface.action.ToolBarManager;

import org.eclipse.jface.dialogs.ProgressMonitorDialog;

import org.eclipse.jface.operation.IRunnableWithProgress;

import org.eclipse.jface.viewers.ColumnPixelData;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TableLayout;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;

import org.eclipse.jface.window.Window;

import org.eclipse.swt.SWT;

import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.ViewForm;

import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;

import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;

import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;

import de.fernunihagen.rtt.framework.ITestActor;
import de.fernunihagen.rtt.framework.ITestDescription;
import de.fernunihagen.rtt.framework.XMLTestConfiguration;

import de.fernunihagen.rtt.framework.RTTCore;

import de.fernunihagen.rtt.framework.oracle.IOracle;

import de.fernunihagen.rtt.framework.repository.IRepository;
import de.fernunihagen.rtt.framework.repository.IRepositoryProvider;

import de.fernunihagen.rtt.framework.testadapter.IRefactoringTestAdapter;
import de.fernunihagen.rtt.framework.testadapter.ReflectiveRefactoringTestAdapter;

import de.fernunihagen.rtt.ui.app.AbstractRTTLaunchConfiguration;
import de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin;

import de.fernunihagen.rtt.ui.internal.util.ViewerField;
import de.fernunihagen.rtt.ui.internal.util.ViewerFieldLabelProvider;

/**
 * Tool test configuration tab.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class ToolTestConfigurationTab extends
		AbstractLaunchConfigurationTab implements
		IToolTestConfigurationChangeListener {

	private static final int ORACLE_COLUMN_INDEX = 1;
	private static final String FIELD = "FIELD";
	private static final String _DEFAULT_TESTADAPTER = "de.fernunihagen.rtt.ui.jdt.testadapter.jquery";
	private static final ViewerField _TESTSUBJECT_RESOURCE = new TestSubjectFieldResource();
	private static final boolean INCLUDE_PLUGIN_PROJECTS = false;

	private ITestActor _activeTestAdapter;
	private Combo _comboActiveTestAdapter;
	private Combo _comboActiveTestBundle;
	private TreeViewer _viewerTestSubject;
	private TreeViewer _viewerTestProject;
	private ToolTestConfiguration _configuration;
	@Invar("null != $target")
	private final List<ViewerField> _fieldTestSubject;
	@Invar("null != $target")
	private final List<ViewerField> _fieldTestProject;

	/**
	 * Creates a tool test configuration tab.
	 */
	public ToolTestConfigurationTab() {
		_fieldTestSubject = new ArrayList<ViewerField>();
		_fieldTestProject = new ArrayList<ViewerField>();
		_viewerTestSubject = null;
		_viewerTestProject = null;
		_configuration = null;
		_comboActiveTestAdapter = null;
		_comboActiveTestBundle = null;
		_activeTestAdapter = null;
	}

	private IOracle[] getOracles() throws CoreException {
		return RTTCore.getOracleProvider().get();
	}

	private IRefactoringTestAdapter[] getTestAdapters() throws CoreException {
		return RTTCore.getRefactoringTestAdapterProvider().get();
	}

	@Post("null != $return")
	private ViewerField[] getTestSubjectVisibleFields() {
		return _fieldTestSubject.toArray(new ViewerField[_fieldTestSubject
				.size()]);
	}

	@Post("null != $return")
	private ViewerField[] getTestProjectVisibleFields() {
		return _fieldTestProject.toArray(new ViewerField[_fieldTestProject
				.size()]);
	}

	private void createTestSubjectColumns(final TreeColumn[] currentColumns) {
		createColumns(_viewerTestSubject, getTestSubjectVisibleFields(),
				currentColumns);
	}

	private void createTestProjectColumns(final TreeColumn[] currentColumns) {
		createColumns(_viewerTestProject, getTestProjectVisibleFields(),
				currentColumns);
	}

	@Pre("null != currentColumns")
	private void createColumns(final TreeViewer viewer,
			final ViewerField[] fields, final TreeColumn[] currentColumns) {
		final Tree tree = viewer.getTree();
		final TableLayout layout = new TableLayout();

		for (int i = 0; i < fields.length; i++) {
			final ViewerField field = fields[i];

			TreeViewerColumn column;
			if (i < currentColumns.length) {
				column = new TreeViewerColumn(viewer, currentColumns[i]);
			} else {
				column = new TreeViewerColumn(viewer, SWT.NONE);
				column.getColumn().setResizable(field.isResizeable());
				column.getColumn().setMoveable(true);
			}

			column.getColumn().setData(FIELD, field);

			// Show the help in the first column
			column.setLabelProvider(new ViewerFieldLabelProvider(field));
			column.getColumn().setText(field.getColumnHeaderText());
			column.getColumn().setToolTipText(field.getColumnTooltipText());
			column.getColumn().setImage(field.getColumnHeaderImage());

			final EditingSupport support = field.getEditingSupport(viewer);
			if (support != null) {
				column.setEditingSupport(support);
			}

			layout.addColumnData(new ColumnPixelData(field
					.getDefaultColumnWidth(tree), true, true));
		}

		// Remove extra columns
		if (currentColumns.length > fields.length) {
			for (int i = fields.length; i < currentColumns.length; i++) {
				currentColumns[i].dispose();
			}
		}

		viewer.getTree().setLayout(layout);
		tree.setLinesVisible(true);
		tree.setHeaderVisible(true);
		tree.layout(true);
	}

	@Post("null != $return")
	private ITreeContentProvider getTestSubjectContentProvider() {
		return new TestSubjectContentProvider();
	}

	@Post("null != $return")
	private ITreeContentProvider getTestProjectContentProvider() {
		return new ToolTestContentProvider();
	}

	@Pre("null != parent")
	@Post("null != $return")
	private Composite createComposite(final Composite parent,
			final int columns, final int hspan, int fill) {
		Composite g = new Composite(parent, SWT.NONE);
		g.setLayout(new GridLayout(columns, false));
		g.setFont(parent.getFont());
		GridData gd = new GridData(fill);
		gd.horizontalSpan = hspan;
		g.setLayoutData(gd);
		return g;
	}

	@Pre("null != parent && null != font")
	@Post("null != $return")
	private Composite createComposite(final Composite parent, final Font font,
			int style, final int columns, int hspan, int fill, int marginwidth,
			int marginheight) {
		Composite g = new Composite(parent, style);
		GridLayout layout = new GridLayout(columns, false);
		layout.marginWidth = marginwidth;
		layout.marginHeight = marginheight;
		g.setLayout(layout);
		g.setFont(font);
		GridData gd = new GridData(fill);
		gd.horizontalSpan = hspan;
		g.setLayoutData(gd);
		return g;
	}

	@Pre("null != parent && null != text")
	@Post("null != $return")
	private Label createLabel(final Composite parent, final String text,
			final int hspan) {
		Label l = new Label(parent, SWT.NONE);
		l.setFont(parent.getFont());
		l.setText(text);
		GridData gd = new GridData();
		gd.horizontalSpan = hspan;
		l.setLayoutData(gd);
		return l;
	}

	@Pre("null != parent")
	private void createTestProjectTreeViewer(final Composite parent,
			final int hspan) {
		final Tree tree = new Tree(parent, SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.MULTI | SWT.FULL_SELECTION | SWT.LEFT_TO_RIGHT
				| SWT.BORDER);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = hspan;
		tree.setLayoutData(gd);

		_viewerTestProject = new TreeViewer(tree);
		_viewerTestProject.getTree().setLinesVisible(true);
		_viewerTestProject.setUseHashlookup(true);
		createTestProjectColumns(new TreeColumn[0]);
		_viewerTestProject.setContentProvider(getTestProjectContentProvider());
		createTestProjectContextMenu();
	}

	@Pre("null != parent")
	private void createTestSubjectTreeViewer(final Composite parent,
			final int hspan) {
		final Tree tree = new Tree(parent, SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.MULTI | SWT.FULL_SELECTION | SWT.LEFT_TO_RIGHT
				| SWT.BORDER);
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = hspan;
		tree.setLayoutData(gd);

		_viewerTestSubject = new TreeViewer(tree);
		_viewerTestSubject.getTree().setLinesVisible(true);
		_viewerTestSubject.setUseHashlookup(true);
		createTestSubjectColumns(new TreeColumn[0]);
		_viewerTestSubject.setContentProvider(getTestSubjectContentProvider());
		createTestSubjectContextMenu();
	}

	@Pre("null != parent")
	private ToolBarManager createToolBarManager(final Composite parent,
			final int hspan) {
		ViewForm viewForm = new ViewForm(parent, SWT.FLAT);
		ToolBar toolBar = new ToolBar(viewForm, SWT.FLAT);
		toolBar.setBackground(parent.getBackground());
		ToolBarManager toolBarManager = new ToolBarManager(toolBar);
		viewForm.setTopRight(toolBar);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = hspan;
		viewForm.setLayoutData(gd);
		return toolBarManager;
	}

	@Pre("null != toolBar")
	@Post("null != $return")
	private ToolItem createToolItemButton(final ToolBar toolBar,
			final String text, final Image image) {
		final ToolItem item = new ToolItem(toolBar, SWT.PUSH);
		item.setToolTipText(text);
		item.setImage(image);
		return item;
	}

	@Pre("null != parent")
	private void createTestSubjectToolBar(final Composite parent,
			final int hspan) {
		final ToolBarManager manager = createToolBarManager(parent, hspan);
		final ToolItem persistProjects = createToolItemButton(
				manager.getControl(), "Persist all Project Configurations",
				RTTCoreUIPlugin.createImage("etool16/save.gif"));
		final ToolItem addProject = createToolItemButton(manager.getControl(),
				"Add Project",
				RTTCoreUIPlugin.createImage("etool16/add_exc.gif"));
		final ToolItem removeProjects = createToolItemButton(
				manager.getControl(), "Remove all",
				RTTCoreUIPlugin.createImage("etool16/removea_exc.gif"));

		persistProjects.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				persistAllProjects();
			}
		});
		addProject.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				addProject();
			}
		});
		removeProjects.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				removeAllProjects();
			}
		});
	}

	private void persistAllProjects() {
		for (TestSubjectContainer container : _configuration
				.getTestSubjectContainer()) {
			persistProject(container);
		}
	}

	private void addProject() {
		final ILabelProvider labelProvider = new JavaElementLabelProvider(
				JavaElementLabelProvider.SHOW_DEFAULT);
		final ElementListSelectionDialog dialog = new ElementListSelectionDialog(
				getShell(), labelProvider);

		dialog.setTitle("Project Selection");
		dialog.setMessage("Select a project to add to configuration.");

		final IProject[] configured = _configuration.getProjects();
		final Set<IProject> excluded = new HashSet<IProject>();
		excluded.addAll(Arrays.asList(configured));

		final IJavaProject[] projects = getSharedProjects(excluded,
				INCLUDE_PLUGIN_PROJECTS);

		dialog.setMultipleSelection(true);
		dialog.setElements(projects);
		dialog.setInitialSelections(projects);

		if (dialog.open() == Window.OK) {
			final Object[] selection = dialog.getResult();
			if (selection.length != 0) {
				final Job job = new UIJob("Adding projects") {
					@Override
					public IStatus runInUIThread(final IProgressMonitor pm) {
						IStatus status = Status.OK_STATUS;
						try {
							SubMonitor monitor = SubMonitor.convert(pm,
									"Adding projects", selection.length);

							for (Object selected : selection) {
								final IJavaProject javaProject = (IJavaProject) selected;
								try {
									javaProject.open(null);
									_configuration.addProject(
											javaProject.getProject(),
											monitor.newChild(1));
									_viewerTestSubject.refresh();
								} catch (JavaModelException e) {
									status = e.getStatus();
								}
							}

						} finally {
							if (null != pm) {
								pm.done();
							}
						}
						return status;

					}
				};

				job.setUser(true);
				job.schedule();
			}
		}
	}

	private void removeAllProjects() {
		_configuration.removeAllProjects();
		_viewerTestSubject.refresh();
	}

	private void removeProject(final TestSubjectContainer element) {
		_configuration.removeProject(element);
		_viewerTestSubject.remove(element);
	}

	private Combo createCombo(final Composite parent, final int style,
			final int hspan, final String[] items) {
		Combo c = new Combo(parent, style);
		c.setFont(parent.getFont());
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = hspan;
		c.setLayoutData(gd);
		if (items != null) {
			c.setItems(items);
		}
		c.setVisibleItemCount(30);
		c.select(0);
		return c;
	}

	@Pre("null != parent")
	private void createTestAdapterCombo(final Composite parent, final int hspan) {
		_comboActiveTestAdapter = createCombo(parent, SWT.DROP_DOWN
				| SWT.READ_ONLY, hspan, null);
		try {
			IRefactoringTestAdapter[] adapters = getTestAdapters();
			for (int i = 0; i < adapters.length; i++) {
				final IRefactoringTestAdapter adapter = adapters[i];
				_comboActiveTestAdapter.setData(String.valueOf(i), adapter);
				_comboActiveTestAdapter.add(adapter.getName(), i);
			}

		} catch (CoreException e) {
			getRTTCoreUIPlugin().logError(e.getLocalizedMessage());
		}

		_comboActiveTestAdapter.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				int index = _comboActiveTestAdapter.getSelectionIndex();
				if (index == -1) {
					return;
				}

				int removeIndex = ORACLE_COLUMN_INDEX;
				if (null != _activeTestAdapter) {
					_configuration.removeActor(_activeTestAdapter);

					while (removeIndex < _fieldTestSubject.size()) {
						final ViewerField field = _fieldTestSubject
								.get(removeIndex);
						if (field.getID().equals(_activeTestAdapter.getID())) {
							_fieldTestSubject.remove(removeIndex);
							break;
						}
						removeIndex++;
					}
				}

				_activeTestAdapter = (ITestActor) _comboActiveTestAdapter
						.getData(String.valueOf(index));
				_fieldTestSubject.add(removeIndex,
						new TestSubjectFieldSelection(_activeTestAdapter));
				_configuration.addActor(_activeTestAdapter);

				createTestSubjectColumns(_viewerTestSubject.getTree()
						.getColumns());

				_comboActiveTestBundle.deselectAll();
				_comboActiveTestBundle.notifyListeners(SWT.Selection, null);

				_viewerTestSubject.update(
						_configuration.getTestSubjectContainer(), null);
			}
		});
	}

	private void createTestBundleCombo(final Composite parent, final int hspan) {
		_comboActiveTestBundle = createCombo(parent, SWT.DROP_DOWN
				| SWT.READ_ONLY, hspan, null);

		IJavaProject[] projects;

		try {
			projects = JavaCore.create(getWorkspace().getRoot())
					.getJavaProjects();
		} catch (JavaModelException e) {
			projects = new IJavaProject[0];
		}

		int i = 0;
		for (IJavaProject project : projects) {
			if (null != ToolTestConfiguration.getBundleId(project)) {
				_comboActiveTestBundle.setData(String.valueOf(i),
						project.getHandleIdentifier());
				_comboActiveTestBundle.add(project.getElementName(), i);
				i++;
			}
		}

		_comboActiveTestBundle.addListener(SWT.Selection, new Listener() {
			@Override
			public void handleEvent(Event event) {
				int index = _comboActiveTestBundle.getSelectionIndex();
				String handleIdentifier = null;
				if (index != -1) {
					handleIdentifier = (String) _comboActiveTestBundle
							.getData(String.valueOf(index));
				}

				updateTests(_activeTestAdapter, handleIdentifier);
			}
		});
	}

	private void updateTests(final ITestActor actor,
			final String elementHandleIdentifier) {
		final IJavaElement element = JavaCore.create(elementHandleIdentifier);

		try {
			_configuration.setTestBundle(actor,
					(null != element) ? element.getJavaProject() : null);
			_viewerTestProject.refresh();
		} catch (CoreException e) {
		}
	}

	@Pre("null != parent")
	private SashForm createSash(final Composite parent, final int orientation,
			final int hspan) {
		GridData gd = new GridData(GridData.FILL_BOTH);
		gd.horizontalSpan = hspan;
		SashForm sash = new SashForm(parent, SWT.SMOOTH);
		sash.setOrientation(orientation);
		sash.setLayoutData(gd);
		sash.setFont(parent.getFont());
		sash.setVisible(true);
		return sash;
	}

	private void createHorizontalSpacer(final Composite comp, final int style,
			final int hspan) {
		Label lbl = new Label(comp, style);
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalSpan = hspan;
		lbl.setLayoutData(gd);
	}

	@Override
	public void createControl(final Composite parent) {
		final Composite comp = createComposite(parent, 1, 1,
				GridData.FILL_HORIZONTAL);
		setControl(comp);

		final SashForm form = createSash(comp, SWT.VERTICAL, 1);

		final Composite maincomp = createComposite(form, form.getFont(),
				SWT.NONE, 2, 1, GridData.FILL_BOTH, 0, 0);
		createLabel(maincomp, "Test Adapter:", 1);
		createTestAdapterCombo(maincomp, 1);
		createLabel(maincomp, "Test-Subject Configurations:", 1);
		createTestSubjectToolBar(maincomp, 1);
		createTestSubjectTreeViewer(maincomp, 2);
		createHorizontalSpacer(maincomp, SWT.SEPARATOR | SWT.SHADOW_OUT
				| SWT.HORIZONTAL, 2);

		final Composite extcomp = createComposite(form, form.getFont(),
				SWT.NONE, 2, 1, GridData.FILL_BOTH, 0, 0);
		createLabel(extcomp, "Test-Project Configuration:", 1);
		createTestBundleCombo(extcomp, 1);
		createTestProjectTreeViewer(extcomp, 2);
	}

	private void createTestSubjectContextMenu() {
		// creating the Menu and registering it
		final MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);

		final Menu menu = menuMgr.createContextMenu(_viewerTestSubject
				.getControl());
		_viewerTestSubject.getControl().setMenu(menu);

		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(final IMenuManager manager) {
				menuMgr.add(new Action("Add Project", RTTCoreUIPlugin
						.getImageDescriptor("etool16/add_exc.gif")) {
					@Override
					public void run() {
						addProject();
					}
				});

				final ISelection selection = _viewerTestSubject.getSelection();
				if (null == selection
						|| !(selection instanceof IStructuredSelection)) {
					return;
				}
				final IStructuredSelection structSel = (IStructuredSelection) selection;
				if (structSel.isEmpty()) {
					return;
				}

				final Set<TestSubjectContainer> rootElements = new HashSet<TestSubjectContainer>();
				final Object[] elements = structSel.toArray();
				for (Object element : elements) {
					final AbstractTestSubject testElement = (AbstractTestSubject) element;
					rootElements.add(testElement.getRoot());
				}

				manager.add(new Separator("Selected"));

				if (rootElements.size() > 1) {
					Action action = new Action(
							"Persist Project Configurations", RTTCoreUIPlugin
									.getImageDescriptor("etool16/save.gif")) {
						@Override
						public void run() {
							for (TestSubjectContainer rootElement : rootElements) {
								persistProject(rootElement);
							}
						}
					};
					menuMgr.add(action);

					action = new Action("Remove Projects", RTTCoreUIPlugin
							.getImageDescriptor("etool16/removea_exc.gif")) {
						@Override
						public void run() {
							for (TestSubjectContainer rootElement : rootElements) {
								removeProject(rootElement);
							}
						}
					};
					menuMgr.add(action);
				}

				if (rootElements.size() > 3) {
					return;
				}

				manager.add(new Separator("Persist"));

				for (TestSubjectContainer rootElement : rootElements) {
					final TestSubjectContainer project = rootElement;
					final String name = project.getJavaElement()
							.getElementName();

					final String label = "Persist " + name + " Configuration";
					manager.add(new Action(label, RTTCoreUIPlugin
							.getImageDescriptor("etool16/save.gif")) {
						@Override
						public void run() {
							persistProject(project);
						}
					});
				}

				manager.add(new Separator("Remove"));

				for (TestSubjectContainer rootElement : rootElements) {
					final TestSubjectContainer project = rootElement;
					final String name = project.getJavaElement()
							.getElementName();

					final String label = "Remove " + name;
					manager.add(new Action(label, RTTCoreUIPlugin
							.getImageDescriptor("etool16/remove_exc.gif")) {
						@Override
						public void run() {
							removeProject(project);
						}
					});
				}
			}
		});
	}

	@Pre("null != project")
	private void persistProject(final TestSubjectContainer project) {
		try {
			_configuration.saveProjectConfiguration(project);
		} catch (CoreException e) {
			getRTTCoreUIPlugin().logError(e.getLocalizedMessage());
		}
	}

	private void createTestProjectContextMenu() {
		// creating the Menu and registering it
		final MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);

		final Menu menu = menuMgr.createContextMenu(_viewerTestProject
				.getControl());
		_viewerTestProject.getControl().setMenu(menu);

		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(final IMenuManager manager) {
				final IAdaptable[] testMembers = _configuration
						.getAllTestMembers();
				if (0 == testMembers.length) {
					return;
				}

				menuMgr.add(new Action("Select All", RTTCoreUIPlugin
						.getImageDescriptor("etool16/sel_obj.gif")) {
					@Override
					public void run() {
						_configuration.addAllTestMember(testMembers);
						_viewerTestProject.update(testMembers, null);
					}
				});

				menuMgr.add(new Action("Deselect All", RTTCoreUIPlugin
						.getImageDescriptor("etool16/desel_obj.gif")) {
					@Override
					public void run() {
						_configuration.removeAllTestMember(testMembers);
						_viewerTestProject.update(testMembers, null);
					}
				});
			}
		});
	}

	@Override
	public String getName() {
		return "Test";
	}

	@Override
	public Image getImage() {
		return RTTCoreUIPlugin.createImage("obj16/test.gif");
	}

	private void setActiveRefactoringTestAdapter(
			final IRefactoringTestAdapter[] adapters, final ITestActor[] actors) {
		for (ITestActor actor : actors) {
			if (actor instanceof IRefactoringTestAdapter) {
				for (int i = 0; i < adapters.length; i++) {
					final IRefactoringTestAdapter adapter = adapters[i];
					if (actor == adapter) {
						_comboActiveTestAdapter.select(i);
						_activeTestAdapter = actor;
						return;
					}
				}
			}
		}
	}

	@Override
	public void initializeFrom(final ILaunchConfiguration configuration) {
		try {
			_configuration = new ToolTestConfiguration(
					configuration.getAttribute(
							AbstractRTTLaunchConfiguration.DESCRIPTION,
							(String) null));
			_configuration.addObserver(this);
		} catch (CoreException e) {
			getRTTCoreUIPlugin().logError(e.getLocalizedMessage());
			return;
		}

		final ProgressMonitorDialog dialog = new ProgressMonitorDialog(null);
		final IRunnableWithProgress runnable = new IRunnableWithProgress() {
			public void run(final IProgressMonitor monitor) {
				try {
					_configuration.initialize(monitor);
				} catch (CoreException e) {
				}
			}
		};

		try {
			dialog.run(true, true, runnable);
		} catch (InvocationTargetException e) {
			getRTTCoreUIPlugin().logError(e.getLocalizedMessage());
		} catch (InterruptedException e) {
			// Nothing to be done
		}

		// Set active test adapter
		_comboActiveTestAdapter.deselectAll();

		final ITestActor[] actors = _configuration.getTestActors();
		try {
			setActiveRefactoringTestAdapter(getTestAdapters(), actors);
		} catch (CoreException e) {
			getRTTCoreUIPlugin().logError(e.getLocalizedMessage());
		}

		// Init test subject tree
		_fieldTestSubject.clear();

		for (ITestActor actor : actors) {
			_fieldTestSubject.add(new TestSubjectFieldSelection(actor));
		}

		_fieldTestSubject.add(0, _TESTSUBJECT_RESOURCE);

		createTestSubjectColumns(_viewerTestSubject.getTree().getColumns());
		_viewerTestSubject.setInput(_configuration);

		_fieldTestProject.clear();
		_fieldTestProject.add(new ToolTestFieldResource(_configuration));
		_fieldTestProject.add(new ToolTestFieldLocation());

		createTestProjectColumns(_viewerTestProject.getTree().getColumns());
		_viewerTestProject.setInput(_configuration);

		// Set active test plug-in bundle
		_comboActiveTestBundle.deselectAll();

		setActiveTestBundle(_activeTestAdapter,
				_configuration.getTestBundleProject(_activeTestAdapter));
	}

	private void setActiveTestBundle(final ITestActor actor,
			final IJavaProject project) {
		if (null == actor || null == project) {
			return;
		}

		for (int i = 0; i < _comboActiveTestBundle.getItemCount(); i++) {
			final String handleIdentifier = (String) _comboActiveTestBundle
					.getData(String.valueOf(i));
			if (project.getHandleIdentifier().equals(handleIdentifier)) {
				_comboActiveTestBundle.select(i);
				updateTests(actor, handleIdentifier);
				break;
			}
		}
	}

	private boolean isValidConfiguration() {
		if (null == _activeTestAdapter) {
			setErrorMessage("Invalid test adapter configuration");
			return false;
		}
		if (0 == _configuration.getProjects().length) {
			setErrorMessage("Missing test subject configuration");
			return false;
		}
		if (0 == _configuration.getTestActors().length) {
			setErrorMessage("Invalid test actor configuration");
			return false;
		}
		if (_activeTestAdapter instanceof ReflectiveRefactoringTestAdapter
				&& null == _configuration
						.getTestBundleProject(_activeTestAdapter)) {
			setErrorMessage("Invalid test project configuration");
			return false;
		}
		if (_activeTestAdapter instanceof ReflectiveRefactoringTestAdapter
				&& 0 == _configuration.getTestMembers().length) {
			setErrorMessage("No tests selected");
			return false;
		}
		return true;
	}

	@Override
	public boolean isValid(final ILaunchConfiguration config) {
		setMessage(null);
		setErrorMessage(null);

		return isValidConfiguration();
	}

	@Post("null != $return")
	private Display getDisplay() {
		Display display;
		display = Display.getCurrent();
		if (display == null) {
			display = Display.getDefault();
		}
		return display;
	}

	@Override
	public void performApply(final ILaunchConfigurationWorkingCopy configuration) {
		BusyIndicator.showWhile(getDisplay(), new Runnable() {
			@Override
			public void run() {
				final String value = _configuration.toString();

				configuration.setAttribute(
						AbstractRTTLaunchConfiguration.DESCRIPTION, value);
			}
		});
	}

	@Post("null != $return")
	private IWorkspace getWorkspace() {
		return getRTTCoreUIPlugin().getWorkspace();
	}

	@Pre("null != excluded")
	@Post("null != $return")
	private IJavaProject[] getSharedProjects(final Set<IProject> excluded,
			final boolean fIncludePlugins) {
		final List<IJavaProject> projects = new ArrayList<IJavaProject>();
		try {
			final IRepositoryProvider provider = RTTCore
					.getRepositoryProvider();

			final IJavaProject[] javaProjects = JavaCore.create(
					getWorkspace().getRoot()).getJavaProjects();

			for (IJavaProject javaProject : javaProjects) {
				try {
					final IProject project = javaProject.getProject();

					if (!excluded.contains(project)
							&& (fIncludePlugins || null == ToolTestConfiguration
									.getBundleId(javaProject))
							&& provider.isSupervised(project)) {

						projects.add(javaProject);
					}
				} catch (CoreException e) {
				}
			}
		} catch (CoreException e) {
		}

		return projects.toArray(new IJavaProject[projects.size()]);
	}

	@Pre("null != excluded")
	@Post("null != $return")
	private Map<IRepository, List<String>> getSharedProjectReferences(
			final Set<IProject> excluded, final boolean fIncludePlugins) {
		final Map<IRepository, List<String>> projects = new HashMap<IRepository, List<String>>();

		try {
			final IRepositoryProvider provider = RTTCore
					.getRepositoryProvider();

			final IJavaProject[] javaProjects = getSharedProjects(excluded,
					fIncludePlugins);

			for (IJavaProject javaProject : javaProjects) {
				final IRepository repository = provider
						.getRepository(javaProject.getProject());

				List<String> list = projects.get(repository);
				if (list == null) {
					list = new ArrayList<String>();
					projects.put(repository, list);
				}

				list.addAll(Arrays.asList(repository.getReferences(
						new IProject[] { javaProject.getProject() }, null)));
			}
		} catch (CoreException e) {
		}

		return projects;
	}

	@Pre("null != projects")
	private int getCountOfProjects(final Map<IRepository, List<String>> projects) {
		int i = 0;
		for (List<String> references : projects.values()) {
			i += references.size();
		}
		return i;
	}

	@Pre("null != projects")
	private int shouldImportSharedProjects(
			final Map<IRepository, List<String>> projects) {
		final int count = getCountOfProjects(projects);
		if (count > 1) {
			final MessageBox messageBox = new MessageBox(getDisplay()
					.getActiveShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			final String message = String
					.format("The workspace contains %d applicable projects, which could be used as test subjects.\n%s\n\n%s",
							count,
							"Do you wish to add ALL of them to configration?",
							"Note: this operation could take same time.");
			messageBox.setText("Confirm Shared Project Import");
			messageBox.setMessage(message.replace('\n', SWT.LF));
			return messageBox.open();
		}
		return SWT.YES;
	}

	@Override
	public void setDefaults(final ILaunchConfigurationWorkingCopy configuration) {
		final XMLTestConfiguration testConfiguration;
		try {
			testConfiguration = XMLTestConfiguration
					.createWriteRoot("configuration");
		} catch (CoreException e) {
			getRTTCoreUIPlugin().logError(e.getLocalizedMessage());
			return;
		}

		final XMLTestConfiguration projectConfigurations = testConfiguration
				.createChild(ITestDescription.TYPE_PROJECTS);

		// FIXME: find plug-in projects with RTT tests in workspace and
		// put them to excluded set - add support for plug-in projects
		final Map<IRepository, List<String>> projects = getSharedProjectReferences(
				new HashSet<IProject>(), INCLUDE_PLUGIN_PROJECTS);
		switch (shouldImportSharedProjects(projects)) {
		case SWT.YES: {
			for (Map.Entry<IRepository, List<String>> project : projects
					.entrySet()) {

				final XMLTestConfiguration provider = projectConfigurations
						.createChild(ITestDescription.TYPE_PROJECTS_PROVIDER);
				provider.put(ITestDescription.KEY_NAME, project.getKey()
						.getID());

				for (String projectId : project.getValue()) {
					final XMLTestConfiguration projectConfiguration = provider
							.createChild(ITestDescription.TYPE_PROJECTS_PROVIDER_PROJECT);

					projectConfiguration.put(ITestDescription.KEY_NAME,
							projectId);
				}
			}
		}
		case SWT.NO:
		default:
			break;
		}

		final XMLTestConfiguration actorConfigurations = testConfiguration
				.createChild(ITestDescription.TYPE_ACTORS);

		final List<ITestActor> actors = new ArrayList<ITestActor>();

		try {
			actors.addAll(Arrays.asList(getOracles()));
		} catch (CoreException e) {
		}

		try {
			actors.add(RTTCore.getRefactoringTestAdapterProvider().get(
					_DEFAULT_TESTADAPTER));
		} catch (CoreException e) {
		}

		for (ITestActor actor : actors) {
			final XMLTestConfiguration actorConfiguration = actorConfigurations
					.createChild(actor.getTypeName());
			actorConfiguration.put(ITestDescription.KEY_NAME, actor.getID());
		}

		final StringWriter writer = new StringWriter();
		testConfiguration.save(writer);
		final String value = writer.toString();

		configuration.setAttribute(AbstractRTTLaunchConfiguration.DESCRIPTION,
				value);
	}

	@Override
	public String getId() {
		return "de.fernunihagen.rtt.ui.launcher.tooltestconfigurationtab"; //$NON-NLS-1$
	}

	@Override
	public void update(final ToolTestConfiguration o,
			final ToolTestConfigurationChangeEvent arg) {
		updateLaunchConfigurationDialog();
	}

	@Override
	public void activated(ILaunchConfigurationWorkingCopy workingCopy) {
		// Nothing to be done
	}

	@Override
	public void deactivated(ILaunchConfigurationWorkingCopy workingCopy) {
		// Nothing to be done
	}
}
