package de.fernunihagen.rtt.ui.internal.jdt.launcher;

import static de.fernunihagen.rtt.internal.util.ClassUtil.getClassUtil;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.IAdaptable;

import org.eclipse.jdt.core.IJavaElement;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.CheckboxCellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ViewerCell;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

import de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin;

import de.fernunihagen.rtt.ui.internal.util.ViewerField;

/**
 * Test field resource.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class ToolTestFieldResource extends ViewerField {

	@Invar("null != $target")
	private final ToolTestConfiguration _configuration;

	/**
	 * Creates a test field label.
	 * 
	 * @param configuration
	 *            Test configuration
	 */
	@Pre("null != configuration")
	public ToolTestFieldResource(final ToolTestConfiguration configuration) {
		_configuration = configuration;
	}

	private final class SelectionEditingSupport extends EditingSupport {
		@Invar("null != $target")
		private final CellEditor _editor;

		@Pre("null != viewer")
		public SelectionEditingSupport(final ColumnViewer viewer) {
			super(viewer);
			_editor = new CheckboxCellEditor(null, SWT.CHECK | SWT.READ_ONLY);
		}

		@Override
		protected boolean canEdit(final Object element) {
			return true;
		}

		@Override
		protected CellEditor getCellEditor(final Object element) {
			return _editor;
		}

		@Override
		protected Object getValue(final Object element) {
			return _configuration.containsTestMember(element);
		}

		@Override
		protected void setValue(final Object element, final Object value) {
			if ((Boolean) value) {
				_configuration.addTestMember((IAdaptable) element);
			} else {
				_configuration.removeTestMember((IAdaptable) element);
			}
			getViewer().update(element, null);
		}
	}

	@Override
	public EditingSupport getEditingSupport(final ColumnViewer viewer) {
		return new SelectionEditingSupport(viewer);
	}

	@Override
	public String getColumnHeaderText() {
		return "Test-Class";
	}

	@Override
	public String getID() {
		return "resource";
	}

	@Override
	public int getDefaultColumnWidth(final Control control) {
		return 4 * super.getDefaultColumnWidth(control);
	}

	private <T> T as(final IAdaptable adaptable, final Class<? extends T> clazz) {
		return getClassUtil().as(adaptable, clazz);
	}

	@Override
	public void update(final ViewerCell cell) {
		final IAdaptable element = (IAdaptable) cell.getElement();
		final IJavaElement javaElement = as(element, IJavaElement.class);

		if (_configuration.containsTestMember(element)) {
			cell.setImage(RTTCoreUIPlugin.createImage("etool16/sel_obj.gif"));
		} else {
			cell.setImage(RTTCoreUIPlugin.createImage("etool16/desel_obj.gif"));
		}

		cell.setText(javaElement.getElementName());
	}

}
