package de.fernunihagen.rtt.ui.internal.util;

import static de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin.getRTTCoreUIPlugin;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.OperationCanceledException;

import org.eclipse.swt.widgets.Display;

import de.fernunihagen.rtt.internal.util.ICommand;

/**
 * A cross thread operation for use of UI-Operations in non-UI-Thread's.
 * 
 * @param <R>
 *            Return type
 * 
 * @author Osama El Hosami
 */
@Contract
public final class CrossThreadOperation<R> implements Runnable {
	private Throwable _ex;
	private R _return;
	@Invar("null != $target")
	private final ICommand<R, ? extends Exception> _command;

	private CrossThreadOperation(final ICommand<R, ? extends Exception> command) {
		_command = command;
	}

	/**
	 * Creates a cross thread operation encapsulating given command.
	 * 
	 * @param <R>
	 *            Return type
	 * 
	 * @param command
	 *            Command
	 * 
	 * @return Cross thread operation
	 */
	public static <R> CrossThreadOperation<R> create(
			final ICommand<R, ? extends Exception> command) {
		return new CrossThreadOperation<R>(command);
	}

	private void reportError() throws CoreException {
		if (hasException()) {
			if (_ex instanceof OperationCanceledException) {
				throw (OperationCanceledException) _ex;
			}

			getRTTCoreUIPlugin().reportError(null, _ex);
		}
	}

	@Post("null != $return")
	private Display getDisplay() {
		Display display;
		display = Display.getCurrent();
		if (display == null) {
			display = Display.getDefault();
		}
		return display;
	}

	private boolean hasException() {
		return null != _ex;
	}

	/**
	 * Retrieves the return value of execution.
	 * 
	 * @return The return value
	 */
	public R getReturn() {
		return _return;
	}

	/**
	 * Executes the cross thread operation.
	 * 
	 * @return Return value
	 * 
	 * @throws Exception
	 *             <ul>
	 *             <li>If an exception occurs</li>
	 *             </ul>
	 */
	public R execute() throws CoreException {
		_return = null;
		_ex = null;

		getDisplay().syncExec(this);
		reportError();

		return getReturn();
	}

	@Override
	public void run() {
		try {
			_return = _command.execute();
		} catch (Throwable e) {
			_ex = e;
		}
	}
}
