package de.fernunihagen.rtt.ui.internal.util;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jface.viewers.EditingSupport;

import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.viewers.ViewerColumn;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;

import org.eclipse.swt.widgets.Control;

/**
 * Viewer field.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class ViewerField {

	/**
	 * Creates a viewer field
	 */
	public ViewerField() {
	}

	/**
	 * Retrieves the font width.
	 * 
	 * @param control
	 *            Control
	 * 
	 * @return width
	 */
	public static int getFontWidth(final Control control) {
		GC gc = new GC(control.getDisplay());
		int width = gc.getFontMetrics().getAverageCharWidth();
		gc.dispose();
		return width;
	}

	/**
	 * Column header text.
	 * 
	 * @return text
	 */
	@Post("null != $return")
	public abstract String getColumnHeaderText();

	/**
	 * Column tool tip text.
	 * 
	 * @return text
	 */
	@Post("null != $return")
	public String getColumnTooltipText() {
		return getColumnHeaderText();
	}

	/**
	 * Retrieves the column header image
	 * 
	 * @return Image
	 */
	public Image getColumnHeaderImage() {
		return null;
	}

	/**
	 * Retrieves the editing support for column.
	 * 
	 * @param viewer
	 *            Viewer
	 * 
	 * @return Editing support
	 */
	public EditingSupport getEditingSupport(final ColumnViewer viewer) {
		return null;
	}

	/**
	 * Retrieves the default column width.
	 * 
	 * @param control
	 *            Control
	 * 
	 * @return Default column width
	 */
	@Pre("null != control")
	public int getDefaultColumnWidth(final Control control) {
		return 15 * getFontWidth(control);
	}

	/**
	 * Retrieves the field identifier.
	 * 
	 * @return Identifier
	 */
	public abstract String getID();

	/**
	 * Tests if field is resizeable.
	 * 
	 * @return True if satisfied otherwise false
	 */
	public boolean isResizeable() {
		return true;
	}

	/**
	 * Updates a cell with field content
	 * 
	 * @param cell
	 *            Cell
	 */
	@Pre("null != cell")
	public abstract void update(ViewerCell cell);

	/**
	 * Dispose of test element field which was used with the given column viewer
	 * and column.
	 * 
	 * @param viewer
	 *            the viewer
	 * @param column
	 *            the column, or <code>null</code> if a column is not available.
	 */
	@Pre("null != viewer")
	public void dispose(ColumnViewer viewer, ViewerColumn column) {

	}
}
