package de.fernunihagen.rtt.ui.internal.view;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.ltk.core.refactoring.RefactoringDescriptorProxy;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Control;

import de.fernunihagen.rtt.framework.ITestContext;
import de.fernunihagen.rtt.framework.ITestElement;

import de.fernunihagen.rtt.framework.model.TestRootContext;
import de.fernunihagen.rtt.framework.testadapter.IRefactoringTestAdapter;
import de.fernunihagen.rtt.framework.testadapter.RefactoringTestElement;
import de.fernunihagen.rtt.ui.internal.util.ViewerField;

/**
 * Test element field label.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TestElementFieldLabel extends ViewerField {

	@Override
	public int getDefaultColumnWidth(final Control control) {
		return 5 * super.getDefaultColumnWidth(control);
	}

	@Override
	@Post
	public String getColumnHeaderText() {
		return "Label";
	}

	@Override
	@Post
	public String getID() {
		return "label";
	}

	private static boolean isRootContextAndActorIsRefactoringTestAdapter(
			final ITestElement element) {
		return element instanceof TestRootContext
				&& ((ITestContext) element).getState().getOriginator() instanceof IRefactoringTestAdapter;
	}

	/**
	 * Retrieves the text representation of given test element.
	 * 
	 * @param element
	 *            Test element
	 * 
	 * @return Textual representation
	 */
	@Pre("null != element")
	@Post("null != $return")
	public static String getText(final ITestElement element) {
		String text = element.getLabel();
		final Object target = element.getTarget();

		if (null != target
				&& isRootContextAndActorIsRefactoringTestAdapter(element)) {

			if (target instanceof ITestElement) {
				text = ((ITestElement) target).getLabel();
			} else {
				text = target.toString();
			}

		} else if (element instanceof RefactoringTestElement) {
			final RefactoringTestElement targetTestElement = (RefactoringTestElement) element;
			final RefactoringDescriptorProxy descriptor = targetTestElement
					.getTarget();
			if (null != descriptor) {
				text = descriptor.toString();
			} else {
				text = targetTestElement.getLabel();
			}
		}

		return text.replace('\n', SWT.LF);
	}

	@Override
	@Pre
	public void update(final ViewerCell cell) {
		final ITestElement element = (ITestElement) cell.getElement();
		cell.setText(getText(element));
	}
}
