package de.fernunihagen.rtt.ui.internal.view;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.viewers.ViewerRow;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Image;

import de.fernunihagen.rtt.framework.ITestElement;
import de.fernunihagen.rtt.framework.ITestElementContainer;
import de.fernunihagen.rtt.framework.ITestElement.Result;
import de.fernunihagen.rtt.framework.model.TestRootContext;
import de.fernunihagen.rtt.framework.testadapter.RefactoringTestElement;

import de.fernunihagen.rtt.ui.internal.RTTCoreUIPlugin;
import de.fernunihagen.rtt.ui.internal.util.ViewerField;

/**
 * Test element field result.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TestElementFieldResult extends ViewerField {

	private static final Color ROOT_ERROR = new Color(RTTCoreUIPlugin
			.getDefault().getWorkbench().getDisplay(), 255, 128, 128);
	private static final Color CHILD_ERROR = new Color(RTTCoreUIPlugin
			.getDefault().getWorkbench().getDisplay(), 255, 180, 180);
	private static final Color ROOT_IGNORED = new Color(RTTCoreUIPlugin
			.getDefault().getWorkbench().getDisplay(), 195, 195, 195);
	private static final Color CHILD_IGNORED = new Color(RTTCoreUIPlugin
			.getDefault().getWorkbench().getDisplay(), 220, 220, 220);
	private static final Color DEFAULT = new Color(RTTCoreUIPlugin.getDefault()
			.getWorkbench().getDisplay(), 255, 255, 255);

	@Override
	@Post
	public String getColumnHeaderText() {
		return "Result";
	}

	@Override
	@Post
	public String getID() {
		return "result";
	}

	private void setBackground(final ViewerRow row, final Color color) {
		for (int i = 0; i < row.getColumnCount(); i++) {
			row.setBackground(i, color);
		}
	}

	private Color getBackgroundError(final ITestElement element) {
		if (element instanceof TestRootContext) {
			return ROOT_ERROR;
		}
		return CHILD_ERROR;
	}

	private Color getBackgroundFailure(final ITestElement element) {
		return DEFAULT;
	}

	private Color getBackgroundIgnored(final ITestElement element) {
		if (element instanceof TestRootContext) {
			return ROOT_IGNORED;
		}
		return CHILD_IGNORED;
	}

	private Image getImageForContainerByResult(final ITestElement.Result result) {
		switch (result) {
		case IGNORED:
			return RTTCoreUIPlugin.createImage("obj16/testignored.gif");
		case OK:
			return RTTCoreUIPlugin.createImage("obj16/tcontainerok.gif");
		case FAILURE:
			return RTTCoreUIPlugin.createImage("obj16/tcontainerfail.gif");
		case ERROR:
			return RTTCoreUIPlugin.createImage("obj16/tcontainererror.gif");
		case NONE:
		case UNDEFINED:
		default:
			return RTTCoreUIPlugin.createImage("obj16/tcontainer.gif");
		}
	}

	private Image getImageForElementByResult(final ITestElement.Result result) {
		switch (result) {
		case IGNORED:
			return RTTCoreUIPlugin.createImage("obj16/testignored.gif");
		case OK:
			return RTTCoreUIPlugin.createImage("obj16/testok.gif");
		case FAILURE:
			return RTTCoreUIPlugin.createImage("obj16/testfail.gif");
		case ERROR:
			return RTTCoreUIPlugin.createImage("obj16/testerr.gif");
		case NONE:
		case UNDEFINED:
		default:
			return RTTCoreUIPlugin.createImage("obj16/test.gif");
		}
	}

	private Image getImage(final ITestElement element,
			final ITestElement.Result result) {
		if (element instanceof ITestElementContainer) {
			return getImageForContainerByResult(result);
		}
		return getImageForElementByResult(result);
	}

	public boolean hasIgnoredChildren(final ITestElement element) {
		if (element instanceof ITestElementContainer) {
			final ITestElementContainer container = (ITestElementContainer) element;
			for (ITestElement child : container.getChildren()) {
				if (hasIgnoredChildren(child)) {
					return true;
				}
			}
		}
		return element.getTestResult() == Result.IGNORED;
	}

	private boolean notExecuted(final ITestElement element) {
		return !(element.getTarget() instanceof RefactoringTestElement);
	}

	@Override
	@Pre
	public void update(final ViewerCell cell) {
		final ITestElement element = (ITestElement) cell.getElement();
		final ITestElement.Result result = element.getTestResult();
		final ViewerRow row = cell.getViewerRow();

		if (result == Result.ERROR) {
			setBackground(row, getBackgroundError(element));
		} else if (result == Result.FAILURE) {
			setBackground(row, getBackgroundFailure(element));
		} else if (hasIgnoredChildren(element) && notExecuted(element)) {
			setBackground(row, getBackgroundIgnored(element));
		} else {
			setBackground(row, DEFAULT);
		}

		row.getCell(0).setImage(getImage(element, result));
		cell.setText(result.name());
	}
}
