package de.fernunihagen.rtt.framework;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Actor life cycle management interface.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface ITestActivator {
	/**
	 * Null life-cycle management handler
	 */
	ITestActivator NULL_ACTIVATOR = new ITestActivator() {
		@Override
		public void start(ITestContext context, IProgressMonitor monitor)
				throws CoreException {
		}

		@Override
		public void stop(ITestContext context) {

		}
	};

	/**
	 * Called when an actor is activated before test run.
	 * 
	 * @param context
	 *            Run context
	 * @param monitor
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != context")
	void start(ITestContext context, IProgressMonitor monitor)
			throws CoreException;

	/**
	 * Called when an actor is deactivated after test run.
	 * 
	 * @param context
	 *            Run context
	 */
	@Pre("null != context")
	void stop(ITestContext context);
}
