package de.fernunihagen.rtt.framework;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Actor interface. Provides basic protocol for test components such as oracles,
 * test adapters or test runners.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface ITestActor {
	/**
	 * Retrieves the actor type name.
	 * 
	 * @return Type name
	 */
	@Post("null != $return")
	String getTypeName();

	/**
	 * Retrieves the actor name.
	 * 
	 * @return Name
	 */
	@Post("null != $return")
	String getName();

	/**
	 * Retrieves the unique identifier.
	 * 
	 * @return Unique identifier
	 */
	@Post("null != $return")
	String getID();

	/**
	 * Retrieves the test actor state factory.
	 * 
	 * @return Test actor state factory
	 */
	@Post("null != $return")
	ITestActorStateFactory getStateFactory();

	/**
	 * Retrieves the life-cycle management handler.
	 * 
	 * @return Life-cycle management handler
	 */
	@Post("null != $return")
	ITestActivator getActivator();

	/**
	 * Retrieves the applicable element collector.
	 * 
	 * @param filter
	 *            Filter
	 * @param scope
	 *            Projects in search scope
	 * 
	 * @return Applicable element collector
	 */
	@Pre("null != filter && null != scope")
	@Post("null != $return")
	IElementCollector<? extends IAdaptable> getCollector(
			IElementFilter<IAdaptable> filter, IProject[] scope);

	/**
	 * Applies the actor actions.
	 * 
	 * @param context
	 *            Test context
	 * @param monitor
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Pre("null != $context")
	void apply(ITestContext context, IProgressMonitor monitor)
			throws CoreException;
}
