package de.fernunihagen.rtt.framework;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

/**
 * Test element container.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface ITestElementContainer extends ITestElement {

	/**
	 * Retrieves the list of child test elements.
	 * 
	 * @return Child test elements
	 */
	@Post("null != $return")
	ITestElement[] getChildren();

	/**
	 * Test if container has children.
	 * 
	 * @return true if container has children
	 */
	boolean hasChildren();

	/**
	 * Sets the test target object.
	 * 
	 * @param target
	 *            Test target object
	 */
	void setTarget(Object target);	
	
	/**
	 * Adds a test element to container
	 * 
	 * @param element
	 *            Test element
	 */
	@Pre("null != element && element != $this && element.getParentContainer() == $this")
	void add(ITestElement element);

	/**
	 * Sets the progress state of run context.
	 * 
	 * @param state
	 *            Progress state
	 */
	@Pre("null != state")
	void setProgressState(ProgressState state);

	/**
	 * Creates a child test element container.
	 * 
	 * @param initSize
	 *            Initial size
	 * @param label
	 *            Label
	 * 
	 * @return Test element container
	 */
	@Pre("0 <= initSize && null != label")
	@Post("null != $return")
	ITestElementContainer createChildContainer(int initSize, String label);

	/**
	 * Creates a child test element.
	 * 
	 * @param label
	 *            Label
	 * @param result
	 *            Test result
	 * 
	 * @return Test element
	 */
	@Pre("null != label")
	@Post("null != $return")
	ITestElement createChild(String label, Result result);

	/**
	 * Sets the test result of container.
	 * 
	 * @param result
	 *            Test result
	 */
	@Pre("null != result")
	void setTestResult(Result result);
}
