package de.fernunihagen.rtt.framework;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;

/**
 * Test run listener registry interface.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface ITestRunListenerRegistry {
	/**
	 * Retrieves the test run listener registry identifier.
	 * 
	 * @return Identifier
	 */
	@Post("null != $return")
	String getID();

	/**
	 * Retrieves all available test run listeners.
	 * 
	 * @return List of test run listeners
	 */
	@Post("null != $return")
	ITestRunListener[] get();

	/**
	 * Retrieves the test run listeners with matching identifier.
	 * 
	 * @param id
	 *            Identifier
	 * 
	 * @return The test run listeners
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If not found</li>
	 *             </ul>
	 */
	@Pre("null != id")
	@Post("null != $return")
	ITestRunListener get(String id) throws CoreException;
}
