package de.fernunihagen.rtt.framework;

import org.contract4j5.contract.Contract;

import org.eclipse.core.runtime.CoreException;

import de.fernunihagen.rtt.framework.oracle.IOracleProvider;
import de.fernunihagen.rtt.framework.oracle.IOracleRunner;

import de.fernunihagen.rtt.framework.repository.IRepositoryProvider;

import de.fernunihagen.rtt.framework.testadapter.IRefactoringTestAdapterProvider;
import de.fernunihagen.rtt.framework.testadapter.IRefactoringTestRunner;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getDefault;

/**
 * RTT core services facade.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class RTTCore {

	private RTTCore() {

	}

	/**
	 * Sets the active test run listener registry.
	 * 
	 * @param name
	 *            Test run listener registry name
	 */
	public static void setTestRunListenerRegistry(final String name) {
		getDefault().setActiveTestRunListenerRegistry(name);
	}

	/**
	 * Sets the active respository provider.
	 * 
	 * @param name
	 *            Respository provider name
	 */
	public static void setRepositoryProvider(final String name) {
		getDefault().setActiveRepositoryProvider(name);
	}

	/**
	 * Sets the active oracle provider.
	 * 
	 * @param name
	 *            Oracle provider name
	 */
	public static void setOracleProvider(final String name) {
		getDefault().setActiveOracleProvider(name);
	}

	/**
	 * Sets the active oracle runner.
	 * 
	 * @param name
	 *            Oracle runner name
	 */
	public static void setOracleRunner(final String name) {
		getDefault().setActiveOracleRunner(name);
	}

	/**
	 * Sets the active refactoring test adapter provider.
	 * 
	 * @param name
	 *            Refactoring test adapter provider name
	 */
	public static void setRefactoringTestAdapterProvider(final String name) {
		getDefault().setActiveRefactoringTestAdapterProvider(name);
	}

	/**
	 * Sets the active refactoring test runner.
	 * 
	 * @param name
	 *            Refactoring test runner name
	 */
	public static void setRefactoringTestRunner(final String name) {
		getDefault().setActiveRefactoringTestRunner(name);
	}

	/**
	 * Retrieves the repository provider.
	 * 
	 * @return Repository provider
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If service unavailable</li>
	 *             <ul>
	 */
	public static IRepositoryProvider getRepositoryProvider()
			throws CoreException {
		return getDefault().getRepositoryProvider();
	}

	/**
	 * Retrieves the oracle provider.
	 * 
	 * @return Oracle provider
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If service unavailable</li>
	 *             <ul>
	 */
	public static IOracleProvider getOracleProvider() throws CoreException {
		return getDefault().getOracleProvider();
	}

	/**
	 * Retrieves the refactoring test adapter provider.
	 * 
	 * @return Refactoring test adapter provider
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If service unavailable</li>
	 *             <ul>
	 */
	public static IRefactoringTestAdapterProvider getRefactoringTestAdapterProvider()
			throws CoreException {
		return getDefault().getRefactoringTestAdapterProvider();
	}

	/**
	 * Retrieves the refactoring test runner.
	 * 
	 * @return Refactoring test runner
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If service unavailable</li>
	 *             <ul>
	 */
	public static IRefactoringTestRunner getRefactoringTestRunner()
			throws CoreException {
		return getDefault().getRefactoringTestRunner();
	}

	/**
	 * Retrieves the oracle runner.
	 * 
	 * @return Oracle runner
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If service unavailable</li>
	 *             <ul>
	 */
	public static IOracleRunner getOracleRunner() throws CoreException {
		return getDefault().getOracleRunner();
	}

	/**
	 * Retrieves the test run listeners registry.
	 * 
	 * @return Test run listeners registry
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If service unavailable</li>
	 *             <ul>
	 */
	public static ITestRunListenerRegistry getTestRunListenerRegistry()
			throws CoreException {
		return getDefault().getTestRunListenerRegistry();
	}
}
