package de.fernunihagen.rtt.framework;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getRTTCorePlugin;

import java.util.Map;

import org.apache.log4j.Logger;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Abstract actor.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class TestActor implements ITestActor, ITestActivator,
		ITestActorStateFactory {
	private final static String PROP_ID = "component.name";
	private final static String PROP_NAME = "name";

	@Invar("null != $target")
	private String _id;
	@Invar("null != $target")
	private String _name;

	private static final class LoggerHolder {
		private static final Logger _DEFAULT = getRTTCorePlugin().getLogger(
				TestActor.class.getName());
	}

	@Post("null != $return")
	private Logger getLogger() {
		return LoggerHolder._DEFAULT;
	}

	/**
	 * Creates an abstract actor.
	 */
	public TestActor() {
		_id = getClass().getName().toLowerCase();
		_name = getClass().getSimpleName().toLowerCase();
	}

	/**
	 * Component activation.
	 * 
	 * @param properties
	 *            Property map
	 */
	protected void activate(final Map<?, ?> properties) {
		_id = (String) properties.get(PROP_ID);

		if (properties.containsKey(PROP_NAME)) {
			_name = (String) properties.get(PROP_NAME);
		}

		if (getLogger().isDebugEnabled()) {
			final String message = String.format("[%s (%s) : activated",
					getID(), properties.toString());
			getLogger().debug(message);
		}
	}

	/**
	 * Retrieves the context state object.
	 * 
	 * @param context
	 *            Test context
	 * 
	 * @return State object
	 */
	@Pre("null != context")
	@Post("null != $return")
	protected ITestActorState getState(final ITestContext context) {
		return context.getState();
	}

	@Override
	@Post
	public String getName() {
		return _name;
	}

	@Override
	@Post
	public String getID() {
		return _id;
	}

	@Post
	@Override
	public ITestActivator getActivator() {
		return this;
	}

	@Override
	@Pre
	public void start(final ITestContext context, IProgressMonitor monitor)
			throws CoreException {
	}

	@Override
	@Pre
	public void stop(final ITestContext context) {
	}

	@Override
	@Post
	public ITestActorStateFactory getStateFactory() {
		return this;
	}
}
