package de.fernunihagen.rtt.framework;

import java.util.List;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;

/**
 * Test actor run state object.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class TestActorRunState extends TestActorState {

	@Invar("null != $target")
	private final IElementCollector<? extends IAdaptable> _collector;

	/**
	 * Creates a test actor state object.
	 * 
	 * @param originator
	 *            Test actor
	 * @param collector
	 *            Collector
	 */
	@Pre("null != originator && null != collector")
	public TestActorRunState(final ITestActor originator,
			final IElementCollector<? extends IAdaptable> collector) {
		super(originator);
		_collector = collector;
	}

	/**
	 * Update targets.
	 * 
	 * @param monitor
	 *            The progress monitor to use for reporting progress to the
	 *            user. It is the caller's responsibility to call done() on the
	 *            given monitor. Accepts <code>null</code>, indicating that no
	 *            progress should be reported and that the operation cannot be
	 *            canceled
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	public void updateTargets(IProgressMonitor monitor) throws CoreException {
		_collector.collect(monitor);
	}

	/**
	 * Retrieves the description used to create a context.
	 * 
	 * @return Test description
	 */
	@Post("null != $return")
	public abstract ITestDescription getDescription();

	/**
	 * Retrieves the immutable list of selected targets.
	 * 
	 * @return Target elements
	 */
	@Post("null != $return")
	public List<? extends IAdaptable> getTargets() {
		return _collector.getMatches();
	}

	/**
	 * Retrieves the projects in scope.
	 * 
	 * @return Projects in scope
	 */
	@Post("null != $return")
	public IProject[] getScope() {
		return _collector.getScope();
	}

	@Override
	@Pre
	public void initialize(final ITestElementContainer parent,
			final IProgressMonitor pm) throws CoreException {
		// nothing to be done
	}
}
