package de.fernunihagen.rtt.framework.model;

import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;
import org.eclipse.core.resources.IProject;

import de.fernunihagen.rtt.framework.ITestContext;
import de.fernunihagen.rtt.framework.ITestElementContainer;
import de.fernunihagen.rtt.framework.ITestRunSession;
import de.fernunihagen.rtt.framework.TestActorRunState;

/**
 * Abstract test context.
 * 
 * @author Osama El Hosami
 */
public abstract class AbstractTestContext extends AbstractTestElementContainer
		implements ITestContext {

	/**
	 * Creates an abstract test context.
	 */
	public AbstractTestContext() {
		super();
	}

	/**
	 * Creates an abstract test context.
	 * 
	 * @param result
	 *            Initial result value
	 */
	public AbstractTestContext(final Result result) {
		super(result, 0);
	}

	@Pre
	@Post
	@Override
	public ITestElementContainer createChildContainer(final int initSize,
			final String label) {

		final boolean fNotStarted = getProgressState(false) == ProgressState.NOT_STARTED;
		final RunStage stage = (fNotStarted) ? RunStage.INIT : RunStage.RUN;

		return new TestRunStageElementContainer(this, initSize, label, stage);
	}

	@Override
	@Post
	public String getLabel() {
		return getState().getOriginator().getName();
	}

	@Override
	@Post
	public IProject[] getScope() {
		if (getState() instanceof TestActorRunState) {
			return ((TestActorRunState) getState()).getScope();
		}
		if (getParentContainer() instanceof ITestContext) {
			return ((ITestContext) getParentContainer()).getScope();
		}
		if (getParentContainer() instanceof ITestRunSession) {
			return ((ITestRunSession) getParentContainer()).getScope();
		}
		return getSession().getScope();
	}
}
