package de.fernunihagen.rtt.framework.model;

import java.util.ArrayList;
import java.util.List;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.IAdaptable;

import de.fernunihagen.rtt.framework.ElementFilterOp;
import de.fernunihagen.rtt.framework.IElementFilter;
import de.fernunihagen.rtt.framework.ITestActor;
import de.fernunihagen.rtt.framework.ITestConfiguration;
import de.fernunihagen.rtt.framework.ITestDescription;
import de.fernunihagen.rtt.framework.ITestElement;
import de.fernunihagen.rtt.framework.ITestElementContainer;
import de.fernunihagen.rtt.framework.ITestRunListener;
import de.fernunihagen.rtt.framework.ITestRunSession;

import de.fernunihagen.rtt.framework.model.TestRunEvent.Type;

import de.fernunihagen.rtt.framework.util.IObservable;

import de.fernunihagen.rtt.internal.util.SubjectAdapter;

/**
 * Test root run session.
 * 
 * @author Osama El Hosami
 */
@Contract
public abstract class TestRootRunSession extends AbstractTestRunSession
		implements
		IObservable<TestRootRunSession, ITestRunListener, TestRunEvent> {

	@Invar("null != $target")
	private final SubjectAdapter<TestRootRunSession, ITestRunListener, TestRunEvent> _adapter;
	private List<IElementFilter<IAdaptable>> _filters;

	/**
	 * Creates a test root run session.
	 */
	public TestRootRunSession() {
		super();
		_adapter = new SubjectAdapter<TestRootRunSession, ITestRunListener, TestRunEvent>(
				this);
		_filters = new ArrayList<IElementFilter<IAdaptable>>();
	}

	@Override
	protected boolean reset() {
		_filters.clear();
		return super.reset();
	}

	@Override
	@Post
	public IElementFilter<IAdaptable> getDirectTargetFilter() {
		IElementFilter<IAdaptable> elementFilter = IElementFilter.NONE;
		for (IElementFilter<IAdaptable> filter : _filters) {
			elementFilter = ElementFilterOp.or(elementFilter, filter);
		}
		return ElementFilterOp.not(elementFilter);
	}

	@Override
	@Pre
	public void addIndirectTargetFilter(
			final IElementFilter<IAdaptable> filter) {
		_filters.add(filter);
	}

	@Override
	@Pre
	public void removeIndirectTargetFilter(
			final IElementFilter<IAdaptable> filter) {
		_filters.remove(filter);
	}

	@Override
	@Post
	public ITestRunSession getSession() {
		return this;
	}

	@Override
	@Post
	public ITestElementContainer getParentContainer() {
		return this;
	}

	@Override
	@Pre
	public void notifyChange(final ITestElement element,
			final ProgressState oldState) {
		if (isDisposed()) {
			return;
		}

		final ProgressState newState = element.getProgressState(false);

		Type eventType = Type.UNDEFINED;

		switch (newState) {
		case NOT_STARTED:
			eventType = Type.RESET;
			break;
		case RUNNING:
			eventType = Type.STARTED;
			break;
		case STOPPED:
		case COMPLETED:
			eventType = Type.FINISHED;
			break;
		}

		_adapter.setChanged();
		notifyObservers(new TestRunEvent(element, eventType));
	}

	@Override
	@Post
	@Pre
	public ITestConfiguration getConfiguration(ITestActor actor) {
		final ITestConfiguration actors = getConfiguration().getChild(
				ITestDescription.TYPE_ACTORS);
		if (null != actors) {
			final String typeName = actor.getTypeName();
			final ITestConfiguration[] configurations = actors
					.getChildren(typeName);

			for (ITestConfiguration configuration : configurations) {
				final String id = configuration.get(ITestDescription.KEY_NAME,
						"");
				if (actor.getID().equals(id)) {
					return configuration;
				}
			}

			return actors;
		}
		return getConfiguration();
	}

	@Override
	public void addObserver(final ITestRunListener o) {
		_adapter.addObserver(o);
	}

	@Override
	public int countObservers() {
		return _adapter.countObservers();
	}

	@Override
	public void deleteObserver(ITestRunListener o) {
		_adapter.deleteObserver(o);
	}

	@Override
	public void deleteObservers() {
		_adapter.deleteObservers();
	}

	@Override
	public boolean hasChanged() {
		return _adapter.hasChanged();
	}

	@Override
	public void notifyObservers() {
		_adapter.notifyObservers();
	}

	@Override
	public void notifyObservers(TestRunEvent arg) {
		_adapter.notifyObservers(arg);
	}
}
