package de.fernunihagen.rtt.framework.oracle;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import de.fernunihagen.rtt.framework.ITestActor;

/**
 * Oracle interface, provides the protocol for validation of a project set
 * before and after a refactoring change operation.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface IOracle extends ITestActor {
	/**
	 * Oracle type enumeration
	 */
	enum Type {
		/**
		 * Syntactic oracle type
		 */
		SYNTACTIC(1),
		/**
		 * Semantic oracle type
		 */
		SEMANTIC(2),
		/**
		 * All types of oracle
		 */
		ALL(3);

		/**
		 * Creates a oracle type
		 * 
		 * @param value
		 *            Numeric representation of type
		 */
		Type(int value) {
			_value = value;
		}

		/**
		 * Tests if other type is kind of this type.
		 * 
		 * @param other
		 *            Other type
		 * 
		 * @return True if satisfied otherwise false
		 */
		@Pre("null != other")
		public boolean isKindOf(final Type other) {
			return (_value & other._value) == other._value;
		}

		/**
		 * Compares another type value with this.
		 * 
		 * @param other
		 *            Other type
		 * 
		 * @return zero if both are equal
		 */
		@Pre("null != other")
		public int compare(final Type other) {
			return _value - other._value;
		}

		private final int _value;
	}

	/**
	 * Retrieves the oracle type.
	 * 
	 * @return Oracle type
	 */
	@Post("null != $return" /* && Type.ALL != $return */)
	Type getType();
}
