package de.fernunihagen.rtt.framework.repository;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getRTTCorePlugin;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;

/**
 * Abstract repository provider.
 * 
 * @author O. El Hosami
 */
@Contract
public abstract class AbstractRepositoryProvider implements IRepositoryProvider {

	/**
	 * Reports a status representing an error message.
	 * 
	 * @param message
	 *            Message or null
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>Always</li>
	 *             </ul>
	 */
	@Pre("null != message")
	protected final void reportError(final String message) throws CoreException {
		getRTTCorePlugin().reportError(message);
	}

	/**
	 * Adds a repository.
	 * 
	 * @param repository
	 *            Repository
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>On misconfiguration</li>
	 *             </ul>
	 */
	@Pre("null != repository")
	abstract public void addRepository(IRepository repository)
			throws CoreException;

	/**
	 * Removes a repository.
	 * 
	 * @param repository
	 *            Repository
	 */
	abstract public void removeRepository(IRepository repository);
}
