package de.fernunihagen.rtt.framework.repository;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getRTTCorePlugin;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;

import org.eclipse.core.runtime.IPath;

import de.fernunihagen.rtt.framework.ITestElementContainer;

import de.fernunihagen.rtt.framework.model.TestElement;

/**
 * Diff test element.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class DiffTestElement extends TestElement {

	@Invar("null != $target")
	private final IDiff _diff;

	@Post("null != $return")
	private static IWorkspace getWorkspace() {
		return getRTTCorePlugin().getWorkspace();
	}

	/**
	 * Creates a diff test element.
	 * 
	 * @param parent
	 *            Parent container
	 * @param diff
	 *            Diff
	 */
	@Pre("null != parent && nul != diff")
	public DiffTestElement(final ITestElementContainer parent, final IDiff diff) {
		super(parent);
		_diff = diff;

		parent.add(this);
		setProgressState(ProgressState.COMPLETED);
	}

	@Override
	public IResource getTarget() {
		return getWorkspace().getRoot().findMember(getResourcePath());
	}

	/**
	 * Retrieves the modified resource path.
	 * 
	 * @return Resource
	 */
	@Post("null != $return")
	public IPath getResourcePath() {
		return _diff.getPath();
	}

	/**
	 * Retrieves a string that describes the difference.
	 * 
	 * @return Difference string
	 */
	@Post("null != $return")
	public String getDiffString() {
		return _diff.toDiffString();
	}

	@Override
	@Post
	public String getLabel() {
		return getResourcePath() + " (" + getDiffString() + ")";
	}

	@Override
	@Post
	public Result getTestResult() {
		return Result.NONE;
	}
}
