package de.fernunihagen.rtt.framework.repository;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IResource;

import org.eclipse.core.runtime.CoreException;

/**
 * Repository provider interface.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface IRepositoryProvider {

	/**
	 * Retrieves the list of supported repositories.
	 * 
	 * @return List of repositories
	 */
	IRepository[] getRepositories();

	/**
	 * Tests if resource is supervised by a repository.
	 * 
	 * @param resource
	 *            Resource
	 * 
	 * @return True if resource is supervised otherwise false
	 * 
	 * @throws CoreException
	 */
	boolean isSupervised(IResource resource) throws CoreException;

	/**
	 * Retrieves the repository for a supervised resource.
	 * 
	 * @param resource
	 *            Resource
	 * 
	 * @return Repository
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If resource not supervised</li>
	 *             <li>If repository unknown</li>
	 *             </ul>
	 */
	@Pre("null != resource")
	@Post("null != $return")
	IRepository getRepository(IResource resource) throws CoreException;

	/**
	 * Retrieves the repository with identifies by given repository identifier.
	 * 
	 * @param id
	 *            Repository identifier
	 * 
	 * @return Repository
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If repository unknown</li>
	 *             </ul>
	 */
	@Pre("null != id")
	@Post("null != $return")
	IRepository getRepository(String id) throws CoreException;

}