package de.fernunihagen.rtt.framework.repository.team;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.IPath;

import org.eclipse.team.core.diff.IDiff;
import org.eclipse.team.core.diff.IThreeWayDiff;
import org.eclipse.team.core.diff.ITwoWayDiff;

import org.eclipse.team.core.diff.provider.TwoWayDiff;

/**
 * Team diff apdapter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class TeamDiff implements
		de.fernunihagen.rtt.framework.repository.IDiff {

	@Invar("null != $target")
	private final ITwoWayDiff _adaptee;

	@Pre("null != diff")
	private TeamDiff(final ITwoWayDiff diff) {
		_adaptee = diff;
	}

	private static ITwoWayDiff toLocal(IDiff diff) {
		if (null != diff) {
			if (diff instanceof IThreeWayDiff) {
				final IThreeWayDiff w3diff = ((IThreeWayDiff) diff);
				if (null != w3diff.getLocalChange()) {
					diff = w3diff.getLocalChange();
				} else {
					diff = new TwoWayDiff(diff.getPath(), IDiff.NO_CHANGE, 0);
				}
			}
		}
		// diffs are either two or three way diffs
		return ((ITwoWayDiff) diff);
	}

	private static ITwoWayDiff toRemote(IDiff diff) {
		if (null != diff) {
			if (diff instanceof IThreeWayDiff) {
				final IThreeWayDiff w3diff = ((IThreeWayDiff) diff);
				if (null != w3diff.getRemoteChange()) {
					diff = w3diff.getRemoteChange();
				} else {
					diff = new TwoWayDiff(diff.getPath(), IDiff.NO_CHANGE, 0);
				}
			} else {
				// two way diff, can't indicate remote change
				diff = new TwoWayDiff(diff.getPath(), IDiff.NO_CHANGE, 0);
			}
		}
		return ((ITwoWayDiff) diff);
	}

	/**
	 * Creates a diff reflecting local changes.
	 * 
	 * @param diff
	 *            Team diff
	 * 
	 * @return Diff
	 */
	public static de.fernunihagen.rtt.framework.repository.IDiff local(
			final IDiff diff) {
		return new TeamDiff(toLocal(diff));
	}

	/**
	 * Creates a diff reflecting remote changes.
	 * 
	 * @param diff
	 *            Team diff
	 * 
	 * @return Diff
	 */
	public static de.fernunihagen.rtt.framework.repository.IDiff remote(
			final IDiff diff) {
		return new TeamDiff(toRemote(diff));
	}

	@Override
	public int getFlags() {
		return _adaptee.getFlags();
	}

	@Override
	public IPath getFromPath() {
		return _adaptee.getFromPath();
	}

	@Override
	public int getKind() {
		return _adaptee.getKind();
	}

	@Override
	public IPath getPath() {
		return _adaptee.getPath();
	}

	@Override
	public IPath getToPath() {
		return _adaptee.getToPath();
	}

	@Override
	public String toDiffString() {
		return _adaptee.toDiffString();
	}

	@Override
	public int hashCode() {
		return _adaptee.hashCode();
	}

	@Override
	public String toString() {
		return _adaptee.toString();
	}

	@Override
	public boolean equals(final Object other) {
		if (other == this) {
			return true;
		}
		if (null == other) {
			return false;
		}
		return other.equals(_adaptee);
	}
}
