package de.fernunihagen.rtt.framework.testadapter;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IProject;

import org.eclipse.core.runtime.IAdaptable;

import de.fernunihagen.rtt.TestClientSession;

import de.fernunihagen.rtt.framework.IElementFilter;
import de.fernunihagen.rtt.framework.ITestConfiguration;
import de.fernunihagen.rtt.framework.ITestRunSession;

/**
 * Refactoring test client session.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class RefactoringTestClientSession extends TestClientSession {

	@Invar("null != $return")
	private final ITestRunSession _session;
	@Invar("null != $return")
	private final RefactoringTestRunState _state;

	/**
	 * Refactoring test client session
	 * 
	 * @param session
	 *            Test run session
	 * @param state
	 *            Refactoring run state
	 */
	@Pre("null != session && null != state")
	public RefactoringTestClientSession(final ITestRunSession session,
			final RefactoringTestRunState state) {
		_session = session;
		_state = state;
	}

	@Override
	@Post
	public String getActorID() {
		return _state.getOriginator().getID();
	}

	@Override
	@Post
	public String getActorName() {
		return _state.getOriginator().getName();
	}

	@Override
	@Post
	public ITestConfiguration getConfiguration() {
		return _session.getConfiguration();
	}

	@Override
	@Post
	public IElementFilter<IAdaptable> getTargetFilter() {
		return _session.getDirectTargetFilter();
	}

	@Override
	@Post
	public IProject[] getScope() {
		return _session.getScope();
	}

	@Override
	@Post
	public RefactoringTestDescription getDescription() {
		return _state.getDescription();
	}

	@Override
	public ITestConfiguration getActorConfiguration() {
		return _session.getConfiguration(_state.getOriginator());
	}

}
