package de.fernunihagen.rtt.framework.testadapter;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.ltk.core.refactoring.RefactoringDescriptorProxy;

import de.fernunihagen.rtt.framework.ITestElementContainer;

import de.fernunihagen.rtt.framework.model.TestElement;

/**
 * Refactoring test element.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class RefactoringTestElement extends TestElement {
	private RefactoringDescriptorProxy _descriptor;
	@Invar("null != $target")
	private final String _label;

	/**
	 * Creates a refactoring test element.
	 * 
	 * @param parent
	 *            Parent container
	 * @param name
	 *            Label
	 */
	@Pre("null != parent && null != name")
	public RefactoringTestElement(final ITestElementContainer parent,
			final String name) {
		super(parent);
		_label = name;

		parent.add(this);
	}

	/**
	 * Sets the test result.
	 * 
	 * @param result
	 *            Test result
	 */
	@Pre("null != result")
	public void setTestResult(final IRefactoringStrategy result) {
		_descriptor = result.getDescriptor();
	}

	@Override
	public RefactoringDescriptorProxy getTarget() {
		return _descriptor;
	}

	@Override
	@Post
	public Result getTestResult() {
		return (null == getTarget()) ? Result.IGNORED : Result.OK;
	}

	@Override
	@Pre
	public void setProgressState(final ProgressState state) {
		super.setProgressState(state);
	}

	@Override
	@Post
	public String getLabel() {
		return _label;
	}
}
