package de.fernunihagen.rtt.framework.util;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

/**
 * Implementors of this interface answer one of the prescribed status codes when
 * asked whether to overwrite a certain path string (which could represent a
 * resource path, a file system path, etc) - Definition based on
 * org.eclipse.ui.dialogs.IOverwriteQuery.
 * 
 * @author Osama El Hosami
 */
@Contract
public interface IOverwriteQuery {

	/**
	 * Status code
	 */
	enum StatusCode {
		/**
		 * Status code indicating the operation should be canceled.
		 */
		CANCEL,
		/**
		 * Status code indicating the entity should not be overwritten, but
		 * operation should not be canceled.
		 */
		NO,
		/**
		 * Status code indicating the entity should not be overwritten, but
		 * operation should not be canceled.
		 */
		YES,
		/**
		 * Status code indicating the entity should be overwritten, and all
		 * subsequent entities should be overwritten without prompting.
		 */
		ALL,
		/**
		 * Status code indicating the entity should not be overwritten, and all
		 * subsequent entities should not be overwritten without prompting.
		 */
		NO_ALL
	}

	/**
	 * Returns a status indicating whether the entity represented by the passed
	 * String should be overwritten.
	 * <p>
	 * This method may be called from a non-UI thread, in which case this method
	 * must run the query in a sync exec in the UI thread, if it needs to query
	 * the user.
	 * </p>
	 * 
	 * @param pathString
	 *            the path representing the entity to be overwritten
	 * 
	 * @return one of the return code constants declared on this interface
	 */
	@Pre("null != pathString")
	@Post("null != $return")
	StatusCode queryOverwrite(String pathString);
}
