package de.fernunihagen.rtt.internal.filter;

import static de.fernunihagen.rtt.internal.util.ClassUtil.getClassUtil;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IResource;

import org.eclipse.core.runtime.IAdaptable;

import de.fernunihagen.rtt.framework.IElementCollector;
import de.fernunihagen.rtt.framework.IElementFilter;

/**
 * Collector resource filter adapter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class CollectorResourceFilterAdapter implements
		IElementFilter<IAdaptable> {
	@Invar("null != $target")
	private final Set<IResource> _matches;

	/**
	 * Creates a collector element filter adapter.
	 * 
	 * @param collector
	 *            Collector
	 */
	@Pre("null != collector")
	public CollectorResourceFilterAdapter(
			final IElementCollector<? extends IAdaptable> collector) {
		List<? extends IAdaptable> matches = collector.getMatches();
		_matches = new HashSet<IResource>(matches.size());
		for (IAdaptable element : matches) {
			_matches.add(as(element, IResource.class));
		}
	}

	private <T> T as(final IAdaptable adaptable, final Class<? extends T> clazz) {
		return getClassUtil().as(adaptable, clazz);
	}

	@Override
	@Pre
	public boolean accept(final IAdaptable element) {
		final IResource resource = as(element, IResource.class);
		return null != resource && _matches.contains(resource);
	}
}
