package de.fernunihagen.rtt.internal.filter;

import static de.fernunihagen.rtt.internal.util.ClassUtil.getClassUtil;

import java.util.HashSet;
import java.util.Set;
import java.util.Arrays;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

import org.eclipse.core.resources.IResource;

import org.eclipse.core.runtime.IAdaptable;

import de.fernunihagen.rtt.framework.IElementFilter;

/**
 * Excluded resource filter.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class ExcludedResourceFilter implements IElementFilter<IAdaptable> {

	@Invar("null != $target")
	private final Set<IResource> _excluded;

	/**
	 * Creates an excluded resource filter.
	 * 
	 * @param excluded
	 *            List of excluded resources
	 */
	@Pre("null != excluded")
	public ExcludedResourceFilter(final IResource[] excluded) {
		_excluded = new HashSet<IResource>(excluded.length);
		_excluded.addAll(Arrays.asList(excluded));
	}

	private <T> T as(final IAdaptable adaptable, final Class<? extends T> clazz) {
		return getClassUtil().as(adaptable, clazz);
	}

	@Override
	public boolean accept(final IAdaptable element) {
		final IResource resource = as(element, IResource.class);
		return !_excluded.contains(resource);
	}
}
