package de.fernunihagen.rtt.internal.jdt;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getRTTCorePlugin;
import static de.fernunihagen.rtt.internal.util.ClassUtil.getClassUtil;

import java.util.ArrayList;
import java.util.List;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import org.eclipse.debug.core.ILaunchConfiguration;

import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;

import org.eclipse.jdt.junit.launcher.JUnitLaunchConfigurationDelegate;

/**
 * JUnit launch configuration.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JUnitLaunchConfiguration extends
		JUnitLaunchConfigurationDelegate {

	/**
	 * ID of JUnit 4 test runner kind.
	 */
	public static final String ID_JUNIT4_TEST_KIND = "org.eclipse.jdt.junit.loader.junit4";

	/**
	 * JUnit launch configuration identifier
	 */
	public static final String ID = "de.fernunihagen.rtt.internal.jdt.junit.launchconfig";

	/**
	 * Launch configuration attribute name for list of test case handles to be
	 * executed, all tests in list has to be located in same project.
	 */
	public static final String ATTR_TESTS = ID + ".TESTS";

	/**
	 * Launch configuration attribute name for keep running.
	 */
	public static final String ATTR_KEEPRUNNING = "org.eclipse.jdt.junit.KEEPRUNNING_ATTR";

	/**
	 * Launch configuration attribute name for test container.
	 */
	public static final String ATTR_TEST_CONTAINER = "org.eclipse.jdt.junit.CONTAINER";

	/**
	 * Launch configuration attribute name for test runner kind.
	 */
	public static final String ATTR_TEST_RUNNER_KIND = "org.eclipse.jdt.junit.TEST_KIND";

	/**
	 * Launch configuration attribute name for failure names.
	 */
	public static final String ATTR_FAILURE_NAMES = "org.eclipse.jdt.junit.FAILURENAMES";

	private <T> T as(final IAdaptable adaptable, final Class<? extends T> clazz) {
		return getClassUtil().as(adaptable, clazz);
	}

	private IStatus createError(final String message, final Throwable e) {
		return getRTTCorePlugin().createError(message, e);
	}

	private void reportError(final IStatus[] statuses, final String message)
			throws CoreException {
		getRTTCorePlugin().reportError(statuses, message);
	}

	@Pre("null != configuration")
	private boolean isValid(final ILaunchConfiguration configuration,
			final IType type) throws CoreException {
		return null != type
				&& type.getJavaProject().equals(getJavaProject(configuration));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jdt.junit.launcher.JUnitLaunchConfigurationDelegate#evaluateTests
	 * (org.eclipse.debug.core.ILaunchConfiguration,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Pre("null != configuration")
	protected IMember[] evaluateTests(final ILaunchConfiguration configuration,
			final IProgressMonitor monitor) throws CoreException {

		final List<IStatus> errors = new ArrayList<IStatus>();
		final List<?> testHandles = configuration.getAttribute(ATTR_TESTS,
				new ArrayList<String>());
		final List<IType> testcases = new ArrayList<IType>();

		for (Object element : testHandles) {
			final String testHandle = element.toString();
			final IType testcase = as(JavaCore.create(testHandle), IType.class);
			if (isValid(configuration, testcase)) {
				testcases.add(testcase);
			} else {
				errors.add(createError("Invalid test case handle '"
						+ testHandle + "'", null));
			}
		}

		if (!errors.isEmpty()) {
			reportError(errors.toArray(new IStatus[errors.size()]),
					"Evaluation failed, at least one test case could not been found");
		}

		return testcases.toArray(new IType[testcases.size()]);
	}
}
