package de.fernunihagen.rtt.internal.jdt;

import static de.fernunihagen.rtt.internal.RTTCorePlugin.getRTTCorePlugin;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.compiler.IProblem;

import de.fernunihagen.rtt.framework.ITestElementContainer;
import de.fernunihagen.rtt.framework.model.TestElement;

/**
 * Test compile problem element.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JavaCompileProblemTestElement extends TestElement {

	@Invar("null != $target")
	private final IProblem _problem;

	/**
	 * Creates a test compile problem element.
	 * 
	 * @param parent
	 *            Parent container
	 */
	@Pre("null != parent && null != problem")
	public JavaCompileProblemTestElement(final ITestElementContainer parent,
			final IProblem problem) {
		super(parent);
		_problem = problem;

		parent.add(this);
		setProgressState(ProgressState.COMPLETED);
	}

	@Override
	public IProblem getTarget() {
		return _problem;
	}

	@Post("null != $return")
	private IWorkspaceRoot getWorkspaceRoot() {
		return getRTTCorePlugin().getWorkspace().getRoot();
	}

	@Post("null != $return")
	private IResource getResource() {
		final String filename = String.valueOf(_problem
				.getOriginatingFileName());
		final IPath path = new Path(filename);
		return getWorkspaceRoot().findMember(path);
	}

	/**
	 * Retrieves the problems associated with given compile problem identifier.
	 * 
	 * @return List of problem markers
	 * 
	 * @throws CoreException
	 *             <ul>
	 *             <li>If an error occurs</li>
	 *             </ul>
	 */
	@Post("null != $return")
	public IMarker[] getProblems() throws CoreException {
		return new IMarker[] { getResource().getMarker(_problem.getID()) };
	}

	@Override
	@Post("null != $return")
	public String toString() {
		final String message = String.format("[%s (/X %s) : %s]",
				getParentContainer(), _problem.getID(), getLabel());
		return message;
	}

	@Override
	@Post
	public Result getTestResult() {
		return Result.FAILURE;
	}

	@Override
	@Post
	public String getLabel() {
		final String filename = String.valueOf(_problem
				.getOriginatingFileName());
		final IPath path = new Path(filename);
		return String.format("%s(%s) %s", path.lastSegment(),
				_problem.getSourceLineNumber(), _problem.getMessage());
	}
}
