package de.fernunihagen.rtt.internal.jdt;

import java.io.Serializable;
import java.util.Comparator;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IType;

import de.fernunihagen.rtt.internal.util.InvertedComparator;

/**
 * Another java element name comparator implementation.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JavaElementNameComparator implements
		Comparator<IJavaElement>, Serializable {

	private static final long serialVersionUID = -3113245986583788574L;

	private static final class JavaElementNameComparatorHolder {
		private static final JavaElementNameComparator _DEFAULT = new JavaElementNameComparator();
	}

	private static final class JavaElementNameComparatorInvertedHolder {
		private static final InvertedComparator<IJavaElement> _DEFAULT = new InvertedComparator<IJavaElement>(
				JavaElementNameComparatorHolder._DEFAULT);
	}

	private JavaElementNameComparator() {

	}

	/**
	 * Retrieves the shared instance.
	 * 
	 * @return The shared instance
	 */
	public static Comparator<IJavaElement> getJavaElementNameComparator() {
		return JavaElementNameComparatorHolder._DEFAULT;
	}

	/**
	 * Retrieves the shared inverted instance.
	 * 
	 * @return The shared inverted instance
	 */
	public static Comparator<IJavaElement> getInvertedJavaElementNameComparator() {
		return JavaElementNameComparatorInvertedHolder._DEFAULT;
	}

	@Pre("null != one && null != other")
	@Post("0 != $return || one.equals(other)")
	@Override
	public int compare(final IJavaElement one, final IJavaElement other) {
		final IType oneType = (IType) one.getAdapter(IType.class);
		final IType otherType = (IType) other.getAdapter(IType.class);

		return oneType.getFullyQualifiedName().compareTo(
				otherType.getFullyQualifiedName());
	}
}
