package de.fernunihagen.rtt.internal.jdt;

import java.io.Serializable;
import java.util.Comparator;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

import org.eclipse.jdt.core.IJavaElement;

import de.fernunihagen.rtt.internal.util.InvertedComparator;

/**
 * Another java element type comparator implementation.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JavaElementTypeComparator implements
		Comparator<IJavaElement>, Serializable {

	private static final long serialVersionUID = 2253287462132286633L;

	private static final class JavaElementTypeComparatorHolder {
		private static final JavaElementTypeComparator _DEFAULT = new JavaElementTypeComparator();
	}

	private static final class JavaElementTypeComparatorInvertedHolder {
		private static final InvertedComparator<IJavaElement> _DEFAULT = new InvertedComparator<IJavaElement>(
				JavaElementTypeComparatorHolder._DEFAULT);
	}

	private JavaElementTypeComparator() {

	}

	/**
	 * Retrieves the shared instance.
	 * 
	 * @return The shared instance
	 */
	public static Comparator<IJavaElement> getJavaElementTypeComparator() {
		return JavaElementTypeComparatorHolder._DEFAULT;
	}

	/**
	 * Retrieves the shared inverted instance.
	 * 
	 * @return The shared inverted instance
	 */
	public static Comparator<IJavaElement> getInvertedJavaElementTypeComparator() {
		return JavaElementTypeComparatorInvertedHolder._DEFAULT;
	}

	private final int compareTypes(final int oneType, final int otherType) {
		return oneType - otherType;
	}

	/**
	 * Note: this comparator imposes orderings that are inconsistent with
	 * equals.
	 * 
	 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
	 */
	@Pre("null != one && null != other")
	@Override
	public int compare(final IJavaElement one, final IJavaElement other) {
		return compareTypes(one.getElementType(), other.getElementType());
	}
}
