package de.fernunihagen.rtt.internal.jdt;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IMember;
import org.eclipse.jdt.core.IType;

import de.fernunihagen.rtt.framework.ITestElementContainer;

import de.fernunihagen.rtt.framework.model.AbstractTestElementContainer;

/**
 * Java test element container.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JavaTestElementContainer extends
		AbstractTestElementContainer {
	@Invar("null != $target")
	private final IJavaElement _element;
	@Invar("null != $target")
	private final ITestElementContainer _parent;

	/**
	 * Creates java test element container. Element has to be added to parent
	 * container.
	 * 
	 * @param parent
	 *            Parent container
	 * @param element
	 *            Java element
	 */
	@Pre("null != parent && null != element")
	public JavaTestElementContainer(final ITestElementContainer parent,
			final IJavaElement element) {
		super();
		_element = element;
		_parent = parent;
	}

	@Override
	public IJavaElement getTarget() {
		return _element;
	}

	@Override
	@Post
	public ITestElementContainer getParentContainer() {
		return _parent;
	}

	@Override
	@Post
	public String getLabel() {
		switch (_element.getElementType()) {
		case IJavaElement.TYPE:
			return ((IType) _element).getFullyQualifiedName();
		case IJavaElement.FIELD:
		case IJavaElement.METHOD:
			return ((IMember) _element).getDeclaringType()
					.getFullyQualifiedName()
					+ "." + ((IMember) _element).getElementName();
		case IJavaElement.JAVA_MODEL:
		case IJavaElement.JAVA_PROJECT:
		case IJavaElement.PACKAGE_FRAGMENT_ROOT:
		case IJavaElement.PACKAGE_FRAGMENT:
		case IJavaElement.COMPILATION_UNIT:
		case IJavaElement.CLASS_FILE:
		case IJavaElement.INITIALIZER:
		case IJavaElement.PACKAGE_DECLARATION:
		case IJavaElement.IMPORT_CONTAINER:
		case IJavaElement.IMPORT_DECLARATION:
		case IJavaElement.LOCAL_VARIABLE:
		case IJavaElement.TYPE_PARAMETER:
		case IJavaElement.ANNOTATION:
		default:
			return _element.getElementName();
		}
	}
}
