package de.fernunihagen.rtt.internal.jdt.oracle;

import java.util.HashSet;
import java.util.Set;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import org.eclipse.jdt.core.IJavaModel;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaModelException;

import org.eclipse.jdt.junit.model.ITestCaseElement;
import org.eclipse.jdt.junit.model.ITestElement.FailureTrace;

import de.fernunihagen.rtt.framework.ITestElementContainer;

import de.fernunihagen.rtt.framework.model.TestElement;

/**
 * JUnit test case element.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class JUnitTestCaseElement extends TestElement {
	@Invar("null != $target")
	private Result _result;
	@Invar("null != $target")
	private final String _testclass;
	@Invar("null != $target")
	private final String _testmethod;
	@Invar("null != $target")
	private final IJavaProject _project;
	@Invar("null != $target")
	private String _trace;

	/**
	 * Creates JUnit test case element. Element has to be added to parent
	 * container.
	 * 
	 * @param parent
	 *            Parent container
	 * @param project
	 *            Project
	 * @param testcase
	 *            Test case
	 */
	@Pre("null != parent && null != project && null != testcase")
	public JUnitTestCaseElement(final ITestElementContainer parent,
			final IJavaProject project, final ITestCaseElement testcase) {
		super(parent);
		_result = Result.UNDEFINED;
		_testclass = testcase.getTestClassName();
		_testmethod = testcase.getTestMethodName();
		_trace = "";
		_project = project;
	}

	/**
	 * Returns the stack frame pointing to the failure reason.
	 * 
	 * @return Stack frame
	 */
	public String getStackFrame() {
		return _trace;
	}

	private String getFailureTrace(final FailureTrace trace) {
		if (null == trace || null == trace.getTrace()) {
			return "";
		}
		return trace.getTrace().trim();
	}

	private final IType findType() {
		// for nested classes...
		final String dottedName = _testclass.replace('$', '.');
		try {
			return internalFindType(_project, dottedName,
					new HashSet<IJavaProject>());
		} catch (JavaModelException e) {
		}
		return null;
	}

	private IType internalFindType(final IJavaProject project,
			final String className, final Set<IJavaProject> visitedProjects)
			throws JavaModelException {

		if (visitedProjects.contains(project))
			return null;

		IType type = project.findType(className);
		if (type != null)
			return type;

		// fix for bug 87492: visit required projects explicitly to also
		// find not exported types
		visitedProjects.add(project);
		final IJavaModel javaModel = project.getJavaModel();
		final String[] requiredProjectNames = project.getRequiredProjectNames();

		for (int i = 0; i < requiredProjectNames.length; i++) {
			IJavaProject requiredProject = javaModel
					.getJavaProject(requiredProjectNames[i]);
			if (requiredProject.exists()) {
				type = internalFindType(requiredProject, className,
						visitedProjects);
				if (type != null)
					return type;
			}
		}

		return null;
	}

	private IMethod findMethod(IType type) {
		if (null == type) {
			return null;
		}

		final IMethod method = type.getMethod(_testmethod, new String[0]);
		if (method != null && method.exists()) {
			return method;
		}

		return null;
	}

	@Override
	public IMethod getTarget() {
		return findMethod(findType());
	}

	/**
	 * Stores the test result.
	 * 
	 * @param result
	 *            Test result
	 * @param trace
	 *            FailureTrace or null
	 */
	@Post("null != result")
	public void setTestResult(final Result result, final FailureTrace trace) {
		_result = result;
		_trace = getFailureTrace(trace);
	}

	@Override
	@Pre
	public void setProgressState(final ProgressState state) {
		super.setProgressState(state);
	}

	@Override
	@Post
	public Result getTestResult() {
		return _result;
	}

	@Override
	@Post
	public String getLabel() {
		return _testclass + "." + _testmethod;
	}
}
