package de.fernunihagen.rtt.internal.runtime;

import java.util.Collections;
import java.util.Comparator;
import java.util.Map;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import de.fernunihagen.rtt.framework.ITestActor;
import de.fernunihagen.rtt.framework.ITestConfiguration;
import de.fernunihagen.rtt.framework.ITestDescription;
import de.fernunihagen.rtt.framework.ITestRunSession;

import de.fernunihagen.rtt.framework.oracle.IOracle;

/**
 * Oracle run session.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class OracleRunnerRunState extends TestRunnerRunState<IOracle> {

	private static ITestConfiguration[] getConfiguration(
			final ITestActor actor, final ITestRunSession session) {
		return session.getConfiguration(actor).getChildren(
				ITestDescription.TYPE_ACTORS_ORACLE);
	}

	/**
	 * Creates an oracle run session.
	 * 
	 * @param originator
	 *            Originator
	 * @param session
	 *            Test run session
	 * @param oracles
	 *            Map of oracles
	 */
	@Pre("null != originator && null != session && null != oracles")
	public OracleRunnerRunState(final ITestActor originator,
			final ITestRunSession session, final Map<String, IOracle> oracles) {
		super(originator, oracles, getConfiguration(originator, session));

		// IMPORTANT sort orcales by type so that runner does first syntactic
		// checks
		Collections.sort(_actors, new Comparator<IOracle>() {
			@Override
			@Pre("null != o1 && null != o2")
			public int compare(final IOracle o1, final IOracle o2) {
				return o1.getType().compare(o2.getType());
			}
		});
	}

	@Override
	@Post
	protected IOracle[] getEmptyArray() {
		return new IOracle[0];
	}

	@Override
	public boolean isActive() {
		return false;
	}
}
