package de.fernunihagen.rtt.internal.testadapter;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;
import org.eclipse.ltk.core.refactoring.Refactoring;

import de.fernunihagen.rtt.framework.testadapter.IRefactoringStrategy;
import de.fernunihagen.rtt.framework.testadapter.RefactoringCheckAll;

/**
 * Refactoring test procedure step.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class RefactoringTestProcedureStep extends RunTestProcedureStep {

	@Invar("null != $target")
	private final FlowDecision _decision;
	@Invar("null != $target")
	private final IRefactoringStrategy _strategy;

	/**
	 * Creates a refactoring test procedure step.
	 * 
	 * @param strategy
	 *            Refactoring strategy
	 * @param decision
	 *            Control flow decision
	 */
	@Pre("null != strategy && null != decision")
	public RefactoringTestProcedureStep(final IRefactoringStrategy strategy,
			final FlowDecision decision) {
		_decision = decision;
		_strategy = strategy;
	}

	/**
	 * Creates a refactoring test procedure step.
	 * 
	 * @param refactoring
	 *            Refactoring
	 * @param decision
	 *            Control flow decision
	 */
	@Pre("null != refactoring && null != decision")
	public RefactoringTestProcedureStep(final Refactoring refactoring,
			final FlowDecision decision) {
		this(new RefactoringCheckAll(refactoring), decision);
	}

	/**
	 * Retrieves the test refactoring strategy.
	 * 
	 * @return Refactoring strategy
	 */
	@Post("null != $return")
	public IRefactoringStrategy getStrategy() {
		return _strategy;
	}

	@Override
	@Post
	public FlowDecision getFlowDecision() {
		return _decision;
	}
}
