package de.fernunihagen.rtt.internal.util;

import java.util.Iterator;
import java.util.NoSuchElementException;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Pre;

/**
 * Generic build in array type iterator. Intend is the equal treatment of build
 * in array types with iterators returned by complex data structures like Java
 * Collections, Lists, Maps or Sets.
 * 
 * @param <T>
 *            Array element type of iterator
 * 
 * @author Osama El Hosami
 */
@Contract
public final class ArrayIterator<T> implements Iterator<T> {
	@Invar("null != $target")
	private final T[] _arr;
	@Invar("$target >= 0" /* && $target <= $this._arr.length */)
	private final int _length;
	@Invar("$target >= 0" /* && $target <= $this.length */)
	private int _i;

	/**
	 * Creates an array iterator.
	 * 
	 * @param arr
	 *            Array to used for iteration
	 */
	@Pre("null != arr")
	public ArrayIterator(final T[] arr) {
		this(arr, 0, arr.length);
	}

	/**
	 * Creates an array iterator.
	 * 
	 * @param arr
	 *            Array to use for iteration
	 * @param start
	 *            Start index of iterator
	 * @param length
	 *            Absolute length of array slice
	 */
	@Pre("null != arr && start < length" /* length <= arr.length */)
	public ArrayIterator(final T[] arr, final int start, final int length) {
		_arr = arr;
		_i = start;
		_length = length;
	}

	@Override
	public boolean hasNext() {
		return inBounds();
	}

	@Override
	public T next() {
		if (!inBounds()) {
			throw new NoSuchElementException();
		}
		return _arr[_i++];
	}

	@Override
	public void remove() {
		throw new UnsupportedOperationException();
	}

	private final boolean inBounds() {
		return _i < _length;
	}
}
