package de.fernunihagen.rtt.internal.util;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

import org.eclipse.core.runtime.IAdaptable;

/**
 * Meta-class related utilities.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class ClassUtil {

	private static final class ClassUtilHolder {
		public static final ClassUtil _DEFAULT = new ClassUtil();
	}

	private ClassUtil() {

	}

	/**
	 * Returns the shared instance
	 * 
	 * @return the shared instance
	 */
	public static ClassUtil getClassUtil() {
		return ClassUtilHolder._DEFAULT;
	}

	/**
	 * Casts object to type of class or returns null
	 * 
	 * @param <T>
	 *            Return type
	 * 
	 * @param clazz
	 *            Class instance to cast object to
	 * @param o
	 *            Object instance
	 * 
	 * @return Object instance or null
	 */
	@Pre("null != clazz")
	public <T> T as(final Object o, final Class<? extends T> clazz) {
		return as(o, clazz, null);
	}

	/**
	 * Casts object to type of class or returns default value
	 * 
	 * @param <T>
	 *            Return type
	 * 
	 * @param clazz
	 *            Class instance to cast object to
	 * @param o
	 *            Object instance
	 * @param defaultValue
	 *            Default value
	 * 
	 * @return Object instance or null
	 */
	@Pre("null != clazz")
	public <T> T as(final Object o, final Class<? extends T> clazz,
			final T defaultValue) {
		return clazz.isInstance(o) ? clazz.cast(o) : defaultValue;
	}

	/**
	 * Retrieves the wrapped instance of adapter designated by given type or
	 * returns null
	 * 
	 * @param <T>
	 *            Return type
	 * 
	 * @param clazz
	 *            Class instance of wrapped object
	 * @param adaptable
	 *            Adaptable instance
	 * 
	 * @return Object instance or null
	 */
	@Pre("null != clazz")
	public <T> T as(final IAdaptable adaptable, final Class<? extends T> clazz) {
		return null != adaptable ? clazz.cast(adaptable.getAdapter(clazz))
				: null;
	}
}
