package de.fernunihagen.rtt.internal.util;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

/**
 * Directory node.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class DirectoryNode extends AbstractFileNode implements
		Iterable<AbstractFileNode> {

	@Invar("null != $target")
	private final List<AbstractFileNode> _childs;

	/**
	 * Create a directory node.
	 * 
	 * @param dir
	 *            Directory handle
	 */
	@Pre("null != file")
	DirectoryNode(final File dir) {
		super(dir);
		final File files[] = dir.listFiles();
		final ArrayList<AbstractFileNode> childs = new ArrayList<AbstractFileNode>(
				files.length);
		for (File file : files) {
			childs.add(AbstractFileNode.create(file));
		}
		childs.trimToSize();
		_childs = childs;
	}

	@Override
	@Pre
	public void accept(final FileNodeVisitor visitor) {
		visitor.visit(this);
	}

	@Override
	@Post("null != $return")
	public Iterator<AbstractFileNode> iterator() {
		return _childs.iterator();
	}
}
