package de.fernunihagen.rtt.internal.util;

import java.util.Map;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

/**
 * Cache interface.
 * 
 * @author Osama El Hosami
 * 
 * @param <K>
 *            Key type
 * @param <V>
 *            Value type
 */
@Contract
public interface ICache<K, V> extends Map<K, V> {
	/**
	 * Retrieves the cache value provider
	 * 
	 * @return Cache value provider
	 */
	@Post("null != $return")
	IFactory<V, ICache<K, V>> getProvider();

	/**
	 * Sets the cache value provider
	 * 
	 * @param provider
	 *            Cache value provider
	 */
	@Pre("null != provider")
	void setProvider(final IFactory<V, ICache<K, V>> provider);

	/**
	 * Unsets the cache value provider.
	 * 
	 * @param provider
	 *            Cache value provider
	 */
	void unsetProvider(final IFactory<V, ICache<K, V>> provider);

	/**
	 * Retrieves the capacity of cache.
	 * 
	 * @return Capacity of cache
	 */
	@Post("0 < $return")
	int getCapacity();
}