package de.fernunihagen.rtt.internal.util;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

/**
 * Conditional command interface.
 * 
 * @param <R>
 *            Result type
 * @param <X>
 *            Exception type
 * 
 * @author Osama El Hosami
 */
@Contract
public interface IConditionalCommand<R, X extends Throwable> {
	/**
	 * Invokes the command for condition true
	 * 
	 * @param condition
	 *            Condition
	 * @param args
	 *            List of arguments
	 * 
	 * @return Result
	 */
	@Pre("null != condition")
	R invoke(final ConditionTrue condition, Object... args) throws X;

	/**
	 * Invokes the command for condition true
	 * 
	 * @param condition
	 *            Condition
	 * @param args
	 *            List of arguments
	 * 
	 * @return Result
	 */
	@Pre("null != condition")
	R invoke(final ConditionFalse condition, Object... args) throws X;
}
