package de.fernunihagen.rtt.internal.util;

/**
 * The delegate interface, representing a reference to an instance or class
 * method - described in the article "A Java Programmer Looks at C# Delegates"
 * by <code>Steven M. Lewis</code> and <code>Wilhelm Fitzpatrick</code>.
 * 
 * @param <R>
 *            Result type
 * @param <X>
 *            Exception type
 * 
 * @author Osama El Hosami
 */
public interface IDelegate<R, X extends Throwable> {

	/**
	 * Invokes the delegate.
	 * 
	 * @param args
	 *            Function arguments
	 * 
	 * @return Result
	 */
	R invoke(Object... args) throws X;
}
