package de.fernunihagen.rtt.internal.util;

import java.io.Serializable;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Pre;

/**
 * A mutable boolean.
 * 
 * @author Osama El Hosami
 */
@Contract
public final class MutableBoolean implements Serializable, Comparable<Object> {
	private static final long serialVersionUID = -5254509328159397898L;

	private boolean _value;

	/**
	 * Creates a mutable boolean with given state.
	 * 
	 * @param value
	 *            State value
	 */
	public MutableBoolean(final boolean value) {
		_value = value;
	}

	/**
	 * Creates a mutable boolean with given state.
	 * 
	 * @param value
	 *            State value
	 */
	@Pre("null != value")
	public MutableBoolean(final Boolean value) {
		_value = value.booleanValue();
	}

	/**
	 * Creates a mutable boolean.
	 */
	public MutableBoolean() {
		this(false);
	}

	/**
	 * Retrieves state of boolean.
	 * 
	 * @return state value
	 */
	public boolean getValue() {
		return _value;
	}

	/**
	 * Stores state of boolean.
	 * 
	 * @param value
	 *            new state value
	 * 
	 * @return old state value
	 */
	public boolean setValue(boolean value) {
		boolean old = _value;
		_value = value;
		return old;
	}

	@Override
	public boolean equals(final Object o) {
		return this == o
				|| (null != o && o instanceof MutableBoolean && ((MutableBoolean) o)
						.equals(this));
	}

	@Pre("null != o")
	private boolean equals(final MutableBoolean o) {
		return o.getValue() == getValue();
	}

	@Override
	@Pre("null != o")
	public int compareTo(final Object o) {
		Boolean b;

		if (o instanceof MutableBoolean) {
			b = Boolean.valueOf(((MutableBoolean) o).getValue());
		} else {
			b = (Boolean) o;
		}
		return Boolean.valueOf(getValue()).compareTo(b);
	}

	@Override
	public int hashCode() {
		return Boolean.valueOf(getValue()).hashCode();
	}
}
