package de.fernunihagen.rtt.internal.util;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

import de.fernunihagen.rtt.framework.util.IPropertyChangeListener;
import de.fernunihagen.rtt.framework.util.IPropertyContainer;
import de.fernunihagen.rtt.framework.util.PropertyChangeEvent;

/**
 * Property container implementation, used to hold configuration of a subject
 * 
 * @author Osama El Hosami
 * 
 * @param <S>
 *            Subject type which associated properties belongs to
 */
@Contract
public final class PropertyContainer<S>
		extends
		Subject<IPropertyContainer<S>, IPropertyChangeListener<S>, PropertyChangeEvent>
		implements IPropertyContainer<S> {

	@Invar("null != _map")
	private final Map<String, Object> _map;
	@Invar("null != _subject")
	private final S _subject;

	/**
	 * Constructs a property container
	 * 
	 * @param subject
	 *            Subject which associated properties belongs to
	 */
	public PropertyContainer(final S subject) {
		_map = Collections.synchronizedMap(new HashMap<String, Object>());
		_subject = subject;
	}

	@Override
	@Pre("null != key")
	public Object getProperty(final String key) {
		return getProperty(key, null);
	}

	@Override
	@Pre("null != key")
	@Post("null == defaultValue || null != $return")
	public Object getProperty(final String key, final Object defaultValue) {
		Object value = _map.get(key);
		return value != null ? value : defaultValue;
	}

	@Override
	@Post("null != $return")
	public String[] getPropertyKeys() {
		return _map.keySet().toArray(new String[_map.keySet().size()]);
	}

	@Override
	public boolean hasProperty(final String key) {
		return key != null && _map.containsKey(key);
	}

	@Override
	@Pre("null != key")
	public Object removeProperty(final String key) {
		return _map.remove(key);
	}

	@Override
	@Pre("null != key && null != value")
	public void setProperty(final String key, final Object value) {
		final Object oldVal = _map.put(key, value);
		setChanged();
		notifyObservers(new PropertyChangeEvent(key, value, oldVal));
	}

	@Override
	@Post("null != $return")
	protected PropertyContainer<S> getThis() {
		return this;
	}

	@Override
	public S getSubject() {
		return _subject;
	}

	@Override
	public void clear() {
		_map.clear();
	}
}
