package de.fernunihagen.rtt.internal.util;

import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import org.contract4j5.contract.Contract;
import org.contract4j5.contract.Invar;
import org.contract4j5.contract.Post;
import org.contract4j5.contract.Pre;

/**
 * Reversed list iterator.
 * 
 * @param <T>
 *            Reversed list iterator
 * 
 * @author Osama El Hosami
 */
@Contract
public final class ReversedList<T> implements Iterable<T> {
	@Invar("null != $target")
	private final List<T> _adaptee;

	/**
	 * Creates a reversed list iterator.
	 * 
	 * @param adaptee
	 *            Reversed list iterator
	 */
	@Pre("null != adaptee")
	public ReversedList(final List<T> adaptee) {
		_adaptee = adaptee;
	}

	@Override
	@Post("null != $return")
	public Iterator<T> iterator() {
		final ListIterator<T> i = _adaptee.listIterator(_adaptee.size());
		return new Iterator<T>() {
			public boolean hasNext() {
				return i.hasPrevious();
			}

			public T next() {
				return i.previous();
			}

			public void remove() {
				i.remove();
			}
		};
	}

	/**
	 * Retrieves the reversed list.
	 * 
	 * @param <T>
	 *            Element type
	 * @param list
	 *            List
	 * 
	 * @return Iterable
	 */
	public static <T> Iterable<T> reversed(final List<T> list) {
		return new ReversedList<T>(list);
	}

	/**
	 * Retrieves the reversed list iterator.
	 * 
	 * @param <T>
	 *            Element type
	 * @param list
	 *            List
	 * 
	 * @return Iterator
	 */
	public static <T> Iterator<T> reversedIterator(final List<T> list) {
		return reversed(list).iterator();
	}

}
